
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include <time.h>

//Constantes
#define Machine 0		// 0=PC, 1=MAC (important car lorsqu'on charge des BMP les octets de poids faible et fort sont inverss selon la machine)
#define NbrMaxImages 30000	//Nbr max d'images qu'il est possible de traiter dans un script

//Types
//Image RGBA : 4 octets par pixel (si A=0 transparent)
typedef struct ImageRGBA
{
	unsigned char * data;
	long largeur,hauteur;
	long x0,y0;		//Position de l'image
} ImageRGBA;

//Liste de blocs pour un ajustage
typedef struct TransfoBlocs
{
	float *deltaX;
	float *deltaY;
	float *Ecart;
	int nbBlocsX,nbBlocsY;
} TransfoBlocs;

//
//
//
//Paramtres du script
//Variables globales dfinies dans le script
float ThetaMin,ThetaMax,ThetaPas;
int GlobalDecalageMinX,GlobalDecalageMaxX,GlobalDecalageMinY,GlobalDecalageMaxY,GlobalDecalagePas;
int LargeurGrille,LargeurRecherche,DecalageMinX,DecalageMaxX,DecalageMinY,DecalageMaxY;
float DecalagePas;
int DimensionLibre;
int OptionQuitter;	//1 si on veut que le programme quitte automatiquement en fin d'excution
	//indique quelle fraction de l'image est prise en compte pour le calcul de la translation
	//ex:4 signifie 1/4 de l'image
int FractionPartieCentraleTranslation; 

	float Epsilon=0.01;

int _nbcomplet=0;
int _nbcompletplus=0;
int _nbcompletmoins=0;
int _nbtotal=0;

//Nom du fichier de script
char NomFichierScript[500];

//Images et paramtres des images
char *NomImageTest[NbrMaxImages];
char *NomImageArrivee[NbrMaxImages];
char *NomImageReference;
char *NomImageCompositage;
long NbrImagesTest;			//Nbr d'images  traiter
long ImageReferenceX0,ImageReferenceY0;
int ReferenceX0Affecte,ReferenceY0Affecte;	//1 si la coordonne a t lue dans le fichier index,0 sinon 
long ImageTestX0[NbrMaxImages],ImageTestY0[NbrMaxImages];
int TestX0Affecte[NbrMaxImages],TestY0Affecte[NbrMaxImages];	//1 si la coordonne a t lue dans le fichier index,0 sinon 

//Fichier contenant les dcalages effectus
FILE *FichierShift;

//Images en cours de traitement
ImageRGBA ImageReference,ImageTest;
ImageRGBA ImageCompositage8bits;
unsigned int *ImageCompositage;

//
//
//
//Fonctions principales du programme
void AjusteCentre(void);
void AjusteCentreTranslation(void);
float EcartAjusteCentreTranslation(ImageRGBA &imgTest,ImageRGBA &imgRef,float DeltaX,float DeltaY);
void AjusteBlocs(void);

void integre(void);

void SauveImageCompositage(const char * Nom,int canal);
void AjouteImageCompositage(ImageRGBA & img);
//Calculs sur les images
ImageRGBA TransfoBlocsImageRGBA(ImageRGBA imgOrigine,TransfoBlocs &TB,int lg,int hg);
ImageRGBA TransfoAffineImageRGBA(ImageRGBA imgOrigine,float Theta,float DeltaX,float DeltaY);
ImageRGBA ExtraitImageRotationRGBA(const ImageRGBA & ImageOrigine,int LargeurExtrait,int HauteurExtrait,float Theta);
ImageRGBA CopieImageRGBA(ImageRGBA & img);
ImageRGBA ZoomImageRGBA(ImageRGBA & img,float zoom);
ImageRGBA TranslationImageRGBA(ImageRGBA & img,float DeltaX,float DeltaY);
float EcartImagesRGBA(const ImageRGBA & img1,const ImageRGBA & img2);
void DeriveImageRGBA(ImageRGBA & img);
ImageRGBA FiltreSNN(ImageRGBA & img0,int rayon);

//Prototypes
void LectureScript(void);

void TesteFiltre(void);

//Chargement d'images : alloue la place ncessaire
int ChargeBMP(ImageRGBA & img,const char * Nom);	//Chargement d'un BMP
//Sauvegarde d'images
void SauveBMP(const ImageRGBA &img,const char * Nom);

//Divers
template <class T>
T max(const T & a,const T & b);
template <class T>
T min(const T & a,const T & b);
void ErreurFatale(void);


//
//
//
//
int main (int _nbargs,char **_args)
{
	int i,x,y;
	float nbdiv;
	char c;
	clock_t time1,time2,timeTotal;
	char NomRVBBmp[500],NomRFit[500],NomVFit[500],NomBFit[500],NomBaseCompositage[500];
	
	/*for (i=0;i<_nbargs;i++){
		printf("%s \n",_args[i]);
	}*/

	if (_nbargs>=2)
	{
		strcpy(NomFichierScript,_args[1]);
		OptionQuitter=1;
		if (_nbargs>2)
		{
			printf("Trop de fichiers : seul le premier fichier pourra tre traite\n");
			printf("Veuillez faire un script pour lancer plusieurs fois MoonStack\n");
		}
	}
	else
	{
		strcpy(NomFichierScript,"ScriptMoonStack2.txt");
		OptionQuitter=0;
	}
	
	printf("**************************************\n");
	printf("****        MoonStack v2.1        ****\n");
	printf("****         Eric Royer           ****\n");
	printf("****        24 Avril 2002         ****\n");
	printf("**************************************\n");
        printf("****        Portage Linux         ****\n");
        printf("****   franck.sicard@3demi.net    ****\n");
	printf("**************************************\n");
	printf("\n");


	timeTotal=clock()/CLOCKS_PER_SEC;

	//Lecture du script
	LectureScript();
	if (NomImageReference==NULL)
	{
		printf("Erreur : pas d'image de reference\n");
		ErreurFatale();
	}
	
	//Initialisation de l'image de rfrence
	ChargeBMP(ImageReference,NomImageReference);
	if (ReferenceX0Affecte==1) ImageReference.x0=ImageReferenceX0;
	else ImageReference.x0=ImageReference.largeur/2;
	if (ReferenceY0Affecte==1) ImageReference.y0=ImageReferenceY0;
	else ImageReference.y0=ImageReference.hauteur/2;
	
	//Si on doit faire un compositage : rservation de place
	if (NomImageCompositage!=NULL)
	{
		//Prevoir une image en 16 bits par composante, 3 couches
		ImageCompositage=new unsigned int[ImageReference.largeur*ImageReference.hauteur*4];
		for (long compteur=0;compteur<ImageReference.largeur*ImageReference.hauteur*4;compteur++) ImageCompositage[compteur]=0;
		//AjouteImageCompositage(ImageReference);
	}		
	
	//Ouverture du fichier qui enregistre les dcalages
	FichierShift=fopen("shift.lst","w");
	
	//Traitement des images
	for (i=1;i<=NbrImagesTest;i++)
	{
		printf("\n");
		printf("**************************************************\n");
		printf("Traitement de l'image %i (%s)\n",i,NomImageTest[i]);
		if (ChargeBMP(ImageTest,NomImageTest[i])==0)
		{
			time1=clock()/CLOCKS_PER_SEC;
			if (TestX0Affecte[i]!=0) ImageTest.x0=ImageTestX0[i];
			else ImageTest.x0=ImageReference.x0;
			if (TestY0Affecte[i]!=0) ImageTest.y0=ImageTestY0[i];
			else ImageTest.y0=ImageReference.y0;
			
			//TesteFiltre();
			
			//Recherche de la meilleure rotation/translation
			if (ThetaMin==ThetaMax) AjusteCentreTranslation();
			else AjusteCentre();

			//printf("Image de sortie: x0=%i y0=%i\n",ImageTest.x0,ImageTest.y0);
			if (LargeurGrille>0) 
			{
				//Ajustage spar des blocs de l'image
				time2=clock()/CLOCKS_PER_SEC;
				AjusteBlocs();
				time2=clock()/CLOCKS_PER_SEC-time2;
				//printf("Temps de calcul (blocs): %u\n",time2);		
			}
			
			time1=clock()/CLOCKS_PER_SEC-time1;
			printf("Temps total de calcul pour l'image: %u secondes\n",time1);
				
			SauveBMP(ImageTest,NomImageArrivee[i]);
			
			if (NomImageCompositage!=NULL)
			{
				AjouteImageCompositage(ImageTest);
			}		
			//Libration de la mmoire
			delete [] ImageTest.data;
			//delete [] ImageArrivee.data;
		}
	}
	
	fclose(FichierShift);
	//Sauvegarde de l'image compositee
	if (NomImageCompositage!=NULL)
	{
		printf("Sauvegarde de l'image compositee : %s\n",NomImageCompositage);
		//creation du nom pour sauvegarde des images fit
		//Au depart nom precise dans le script
		strcpy(NomBaseCompositage,NomImageCompositage);
		//Suppression de l'extension si elle tait prsente
		char *ExtensionCompositage=strchr(NomBaseCompositage,'.');
		if (ExtensionCompositage!=NULL)
		{	
			ExtensionCompositage[0]=0;
		}
		strcpy(NomRFit,NomBaseCompositage);
		strcat(NomRFit,"R.fit");
		strcpy(NomVFit,NomBaseCompositage);
		strcat(NomVFit,"V.fit");
		strcpy(NomBFit,NomBaseCompositage);
		strcat(NomBFit,"B.fit");		
		SauveImageCompositage(NomRFit,2);
		SauveImageCompositage(NomVFit,1);
		SauveImageCompositage(NomBFit,0);
		ImageCompositage8bits.x0=ImageReference.x0;
		ImageCompositage8bits.y0=ImageReference.y0;
		ImageCompositage8bits.largeur=ImageReference.largeur;
		ImageCompositage8bits.hauteur=ImageReference.hauteur;
		ImageCompositage8bits.data=new unsigned char[ImageCompositage8bits.largeur*ImageCompositage8bits.hauteur*4];
		for (y=0;y<ImageCompositage8bits.hauteur;y++) 
			for (x=0;x<ImageCompositage8bits.largeur;x++)
			{
				//nbdiv=ImageCompositage[(x+y*ImageReference.largeur)*4+3];
				nbdiv=NbrImagesTest;
				ImageCompositage8bits.data[(x+y*ImageCompositage8bits.largeur)*4]=(ImageCompositage[(x+y*ImageReference.largeur)*4]/nbdiv);
				ImageCompositage8bits.data[(x+y*ImageCompositage8bits.largeur)*4+1]=(ImageCompositage[(x+y*ImageReference.largeur)*4+1]/nbdiv);
				ImageCompositage8bits.data[(x+y*ImageCompositage8bits.largeur)*4+2]=(ImageCompositage[(x+y*ImageReference.largeur)*4+2]/nbdiv);
				ImageCompositage8bits.data[(x+y*ImageCompositage8bits.largeur)*4+3]=255;
			}
		delete [] ImageCompositage;
		strcpy(NomRVBBmp,NomBaseCompositage);
		strcat(NomRVBBmp,".bmp");
		SauveBMP(ImageCompositage8bits,NomRVBBmp);
		delete [] ImageCompositage8bits.data;
	}
	
	delete [] ImageReference.data;
	delete [] NomImageReference;
	for (i=0;i<NbrMaxImages;i++)
	{
		delete [] NomImageTest[i];
		delete [] NomImageArrivee[i];
	}
	printf("\n");
	timeTotal=clock()/CLOCKS_PER_SEC-timeTotal;
	printf("Temps pour l'ensemble du calcul: %u secondes\n",timeTotal);		

	printf("Fin de l'execution de MoonStack\n");
	if (OptionQuitter==0)
	{
		printf("Appuyez sur entree pour quitter\n");
		c=getchar();
	}
	
	return 0;
}

void integre(void)
{
	ImageRGBA img;
	float *imf;
	long x,y;
	float mini=0;
	float maxi=0;
	
	img.largeur=ImageReference.largeur;
	img.hauteur=ImageReference.hauteur;
	img.data=new unsigned char[img.largeur*img.hauteur*4];

	imf=new float[img.largeur*img.hauteur];
	
	//
	for (x=0;x<img.largeur;x++)
	{
		imf[x]=0;
		for (y=1;y<img.hauteur;y++)
		{
			imf[x+y*img.largeur]=img.data[x+(y-1)*img.largeur]+(128-ImageReference.data[(x+y*img.largeur)*4])/10;
			mini=min(mini,imf[x+y*img.largeur]);
			maxi=max(maxi,imf[x+y*img.largeur]);
		}
	}
	//
	for (x=0;x<img.largeur;x++)
	{
		for (y=0;y<img.hauteur;y++)
		{
			img.data[(x+y*img.largeur)*4]=255/(maxi-mini)*imf[x+y*img.largeur]+255*mini/(mini-maxi);
			img.data[(x+y*img.largeur)*4+1]=img.data[(x+y*img.largeur)*4];		
			img.data[(x+y*img.largeur)*4+2]=img.data[(x+y*img.largeur)*4];		
			img.data[(x+y*img.largeur)*4+3]=255;		
		}
	}
	
	
	SauveBMP(img,"integre.bmp");
	delete [] img.data;
}

void TesteFiltre(void)
{
	ImageRGBA img;
	int r;
	char nom[80];
	
	for (r=1;r<4;r++)
	{
		printf("SNN %i\n",r);
		img=FiltreSNN(ImageTest,r);
		sprintf(nom,"SNN%i.bmp",r);
		SauveBMP(img,nom);
		delete [] img.data;
	}
	r=6;
		printf("SNN %i\n",r);
		img=FiltreSNN(ImageTest,r);
		sprintf(nom,"SNN%i.bmp",r);
		SauveBMP(img,nom);
		delete [] img.data;
}

void AjusteCentre(void)
{	
	int DeltaX,DeltaY;
	float Theta;
	float EC,EC1,EC2,EcartMin;
	int i;
	int pourcentageAffiche,pourcentage;
	int NbEtapesTheta;
	ImageRGBA ImgCentre,ImgCentreTranslatee;
	float MeilleurTheta;
	int MeilleurDeltaX,MeilleurDeltaY;
	float t,t1,t2,t3;
	clock_t time1,time2;
			
	printf("Ajustement global : \n");

	MeilleurTheta=0;
	MeilleurDeltaX=0;
	MeilleurDeltaY=0;

	if (ThetaPas==0)
	{
		printf("Erreur : resolution angulaire nulle");
		ErreurFatale();
	}
	NbEtapesTheta=(ThetaMax-ThetaMin)/ThetaPas;
	if (NbEtapesTheta<0) NbEtapesTheta=-NbEtapesTheta;
	
	EcartMin=100000000.*100000000.*100000000.;
		
	time1=clock()/CLOCKS_PER_SEC;

	//Indicateur de progression
	printf("--------------------------------------------------\n");
	pourcentageAffiche=0;

	//Premiere phase: ajustage par pas de 1 degr
	i=0;
	for (Theta=ThetaMin;Theta<=ThetaMax;Theta+=3.141592654/180)
	{
		i++;
		//Indicateur de progression
		pourcentage=(Theta-ThetaMin)/(ThetaMax-ThetaMin)*50/3;
		pourcentage=pourcentage-pourcentageAffiche+1;
		for (i=0;i<pourcentage;i++)
		{
			printf("*");
			pourcentageAffiche++;
		}
		fflush(stdout);

		//Gnration d'une image prtransforme.
		//supprimer la rotation si ThetaMin=ThetaMax
		if (ThetaMin!=ThetaMax) ImgCentre=ExtraitImageRotationRGBA(ImageTest,ImageTest.largeur/4,ImageTest.hauteur/4,Theta);
		else ImgCentre=ExtraitImageRotationRGBA(ImageTest,ImageTest.largeur/FractionPartieCentraleTranslation,ImageTest.hauteur/FractionPartieCentraleTranslation,Theta);
		
		for (DeltaX=GlobalDecalageMinX;DeltaX<=GlobalDecalageMaxX;DeltaX+=GlobalDecalagePas)
		{
			for (DeltaY=GlobalDecalageMinY;DeltaY<=GlobalDecalageMaxY;DeltaY+=GlobalDecalagePas)
			{
				ImgCentreTranslatee.data=ImgCentre.data;
				ImgCentreTranslatee.largeur=ImgCentre.largeur;
				ImgCentreTranslatee.hauteur=ImgCentre.hauteur;
				ImgCentreTranslatee.x0=ImgCentre.x0+DeltaX;
				ImgCentreTranslatee.y0=ImgCentre.y0+DeltaY;
				
				EC=EcartImagesRGBA(ImgCentreTranslatee,ImageReference);
				if (EC<EcartMin)
				{
					EcartMin=EC;
					MeilleurTheta=Theta;
					MeilleurDeltaX=DeltaX;
					MeilleurDeltaY=DeltaY;
					//printf("Theta=%f DeltaX=%i DeltaY=%i Diff=%f\n",MeilleurTheta*180/3.141592654,MeilleurDeltaX,MeilleurDeltaY,EcartMin);
				}
			}
			//printf("DeltaX=%i DeltaY=%i EcartMin=%f\n",DeltaX,DeltaY,EcartMin);
		}
		delete [] ImgCentre.data;
		//printf("Theta=%f Diff=%f\n",Theta*180/3.141592654,EC);

		//printf("%i\n",i);
	}
	time1=clock()/CLOCKS_PER_SEC-time1;
	//printf("Phase 1 : theta=%f  DeltaX=%i   DeltaY=%i\n",MeilleurTheta*180/3.141592654,MeilleurDeltaX,MeilleurDeltaY);
	//printf("Temps Phase1 : %u\n",time1);
	
	if (ThetaMin!=ThetaMax)
	{
		//Deuxime phase: (uniquement pour les rotations) 
		// Recherche du minimum  proximit de la solution approche de la phase 1
		// La fonction a l'air convexe autour de ce point
		//Trichotomie
		time2=clock()/CLOCKS_PER_SEC;
		i=0;
		t=MeilleurTheta-(3.141592654/180*1/2+ThetaPas);
		t3=MeilleurTheta+(3.141592654/180*1/2+ThetaPas);
		while (t3-t>ThetaPas/2)
		{
			//Indicateur de progression
			printf("***");
			fflush(stdout);
			//Mise a jour des bornes
			t1=t+(t3-t)/3;
			t2=t+2*(t3-t)/3;
			//Optimisation:gnrer une ligne sur 2 ou sur 3
			ImgCentre=ExtraitImageRotationRGBA(ImageTest,ImageTest.largeur,ImageTest.hauteur,t1);
			EC1=EcartImagesRGBA(ImgCentre,ImageReference);
			delete [] ImgCentre.data;
			ImgCentre=ExtraitImageRotationRGBA(ImageTest,ImageTest.largeur,ImageTest.hauteur,t2);
			EC2=EcartImagesRGBA(ImgCentre,ImageReference);
			delete [] ImgCentre.data;
			if (EC2<EC1)
			{
				t=t1;
				MeilleurTheta=t1;
				EC=EC1;
			}
			else
			{
				t3=t2;
				MeilleurTheta=t2;
				EC=EC2;
			}
			//printf("Etape %i Theta=%f Ecart prcdant=%f\n",i,MeilleurTheta*180/3.141592654,EC);
			i++;
		}	
		printf("\n");
		printf("theta=%f  DeltaX=%i   DeltaY=%i\n",MeilleurTheta*180/3.141592654,MeilleurDeltaX,MeilleurDeltaY);
		time2=clock()/CLOCKS_PER_SEC-time2;
		//printf("Temps Phase2 : %u  ",time2);
	}
	else printf("\nDeltaX=%i   DeltaY=%i\n",MeilleurDeltaX,MeilleurDeltaY);
	//printf("(Temps Phase1 : %u)\n",time1);
	
	//Ecriture dans le fichier Shift.lst
	if ((GlobalDecalageMinX!=GlobalDecalageMaxX) || (GlobalDecalageMinY!=GlobalDecalageMaxY))
	{
		fprintf(FichierShift,"%i %i ",MeilleurDeltaX,MeilleurDeltaY);
		if ((MeilleurDeltaX==GlobalDecalageMaxX) || (MeilleurDeltaX==GlobalDecalageMinX) || (MeilleurDeltaY==GlobalDecalageMaxY) || (MeilleurDeltaY==GlobalDecalageMinY))
		{
			printf("\n!!! Attention : la valeur maximum de decalage autorise a ete atteinte.\n");
			fprintf(FichierShift,"  Non optimal");
		}
		fprintf(FichierShift,"\n");
	}
	
	//Transformation de l'image test en fonction des paramtres trouvs.
	ImgCentre=TransfoAffineImageRGBA(ImageTest,MeilleurTheta,MeilleurDeltaX,MeilleurDeltaY);
	delete [] ImageTest.data;
	ImageTest=ImgCentre;
}

//Ajustage du centre uniquement en translation
void AjusteCentreTranslation(void)
{	
	int it,DeltaX,DeltaY;
	int nbPas,k;
	float dx,dy,EC,EC0,EC1,EC2,EcartMin,pas,variation,GradX,GradY;
	int pourcentageAffiche;
	ImageRGBA ImgCentre,ImgCentreTranslatee,ImgRefReduite;
	int MeilleurDeltaX,MeilleurDeltaY;
	clock_t time1;

	float ZoomTranslationGlobale=0.125;
			
	printf("Recherche de la meilleure translation : \n");

	MeilleurDeltaX=0;
	MeilleurDeltaY=0;
	
	EcartMin=100000000.*100000000.*100000000.;
		
	time1=clock()/CLOCKS_PER_SEC;

	//Indicateur de progression
	printf("--------------------------------------------------\n");
	pourcentageAffiche=0;

	//Gnration d'une image en taille rduite pour la premire phase de l'ajustement
	ImgCentre=ZoomImageRGBA(ImageTest,ZoomTranslationGlobale);
	ImgRefReduite=ZoomImageRGBA(ImageReference,ZoomTranslationGlobale);
	
	//Recherche de la translation sur l'image taille rduite
	for (DeltaX=GlobalDecalageMinX*ZoomTranslationGlobale;DeltaX<=GlobalDecalageMaxX*ZoomTranslationGlobale;DeltaX+=1)
	{
		for (DeltaY=GlobalDecalageMinY*ZoomTranslationGlobale;DeltaY<=GlobalDecalageMaxY*ZoomTranslationGlobale;DeltaY+=1)
		{
			ImgCentreTranslatee.data=ImgCentre.data;
			ImgCentreTranslatee.largeur=ImgCentre.largeur;
			ImgCentreTranslatee.hauteur=ImgCentre.hauteur;
			ImgCentreTranslatee.x0=ImgCentre.x0+DeltaX;
			ImgCentreTranslatee.y0=ImgCentre.y0+DeltaY;
			
			EC=EcartImagesRGBA(ImgCentreTranslatee,ImgRefReduite);
			if (EC<EcartMin)
			{
				EcartMin=EC;
				MeilleurDeltaX=DeltaX;
				MeilleurDeltaY=DeltaY;
				//printf("DeltaX=%i DeltaY=%i Diff=%f\n",MeilleurDeltaX,MeilleurDeltaY,EcartMin);
			}
		}
		//printf("DeltaX=%i DeltaY=%i EcartMin=%f\n",DeltaX,DeltaY,EcartMin);
	}
	delete [] ImgCentre.data;
	delete [] ImgRefReduite.data;

	time1=clock()/CLOCKS_PER_SEC-time1;
	printf("(Temps Phase1 : %u)\n",time1);
	
	//Recherche au voisinage de cette premire solution trouve
	//algorithme de gradient  faire
	
	//bidouille pas belle sur les variables globales
	//a remplacer par des passages de parametres
int grossebidouille=1;
	if (grossebidouille==1)
	{
		int a1=GlobalDecalageMinX;
		int a2=GlobalDecalageMaxX;
		int a3=GlobalDecalageMinY;
		int a4=GlobalDecalageMaxY;
		GlobalDecalageMinX=(MeilleurDeltaX-1)/ZoomTranslationGlobale-2;
		GlobalDecalageMaxX=(MeilleurDeltaX+1)/ZoomTranslationGlobale+2;
		GlobalDecalageMinY=(MeilleurDeltaY-1)/ZoomTranslationGlobale-2;
		GlobalDecalageMaxY=(MeilleurDeltaY+1)/ZoomTranslationGlobale+2;
		AjusteCentre();
		GlobalDecalageMinX=a1;
		GlobalDecalageMaxX=a2;
		GlobalDecalageMinY=a3;
		GlobalDecalageMaxY=a4;
	}
	
	else
	{
		//algo de gradient

	//rtablir la division
		ImgCentre=ExtraitImageRotationRGBA(ImageTest,ImageTest.largeur/(1+0*FractionPartieCentraleTranslation),ImageTest.hauteur/(1+0*FractionPartieCentraleTranslation),0);
		dx=MeilleurDeltaX/ZoomTranslationGlobale;
		dy=MeilleurDeltaY/ZoomTranslationGlobale;
		pas=0.2;
		variation=pas/10;
		it=0;
		while (pas>0.002)
		{
			it++;
			//calcul du gradient
			EC=EcartAjusteCentreTranslation(ImgCentre,ImageReference,dx,dy);
			printf("iteration %i dx=%f dy=%f ecart=%f\n",it,dx,dy,EC);
			EC1=EcartAjusteCentreTranslation(ImgCentre,ImageReference,dx+variation,dy);
			EC2=EcartAjusteCentreTranslation(ImgCentre,ImageReference,dx,dy+variation);
			GradX=(EC1-EC)/variation;
			GradY=(EC2-EC)/variation;
			printf("EC=%f EC1=%f EC2=%f\n",EC,EC1,EC2);
			printf("GradX=%f GradY=%f\n",GradX,GradY);
			
			
			//Recherche de la meilleure solution possible dans la direction du gradient
			EcartMin=EC;
			nbPas=0;
			for (k=0;k<5;k++)
			{
				EC0=EcartAjusteCentreTranslation(ImgCentre,ImageReference,dx-k*pas*GradX,dy-k*pas*GradY);
				if (EC0<EcartMin)
				{
					EcartMin=EC0;
					nbPas=k;
				}
			}
			//Si le minimum trouv est le point de dpart alors c'est que le pas est trop grand
			if (nbPas==0)
			{
				pas=pas/2;
				printf("Nouveau pas:%f\n",pas);
				variation=variation/2;
			}
			//Mise  jour de dx et dy
			dx=dx-nbPas*pas*GradX;
			dy=dy-nbPas*pas*GradY;		
		}
		delete [] ImgCentre.data;
		MeilleurDeltaX=dx;
		MeilleurDeltaY=dy;
		
		printf("\nDeltaX=%i   DeltaY=%i\n",MeilleurDeltaX,MeilleurDeltaY);
		
		//Ecriture dans le fichier Shift.lst
		if ((GlobalDecalageMinX!=GlobalDecalageMaxX) || (GlobalDecalageMinY!=GlobalDecalageMaxY))
		{
			fprintf(FichierShift,"%i %i ",MeilleurDeltaX,MeilleurDeltaY);
			fprintf(FichierShift,"\n");
		}
		
		//Transformation de l'image test en fonction des paramtres trouvs.
		ImgCentre=TranslationImageRGBA(ImageTest,MeilleurDeltaX,MeilleurDeltaY);
		delete [] ImageTest.data;
		ImageTest=ImgCentre;
	}
	
}
float EcartAjusteCentreTranslation(ImageRGBA &imgTest,ImageRGBA &imgRef,float DeltaX,float DeltaY)
{
	ImageRGBA img;
	float ecart;
	
	img=TranslationImageRGBA(imgTest,DeltaX,DeltaY);
	ecart=EcartImagesRGBA(img,imgRef);
	delete [] img.data;
	
	return ecart;
}

//
//Calcul du meilleur dcalage pour un bloc donn
//Cas de dcalage en float
//MeilleurDeltaX et MeilleurDeltaY valeurs pour le retour du meilleur dcalage
//xb,yb coordonnes du centre du bloc  ajuster
//La fonction retourne la valeur de l'erreur commise
float MeilleurDecalage(float &MeilleurDeltaX,float &MeilleurDeltaY,int xb,int yb)
{
	float DeltaX,DeltaY;
	float EcartMin;
	int x,y,x0i,y0i;
	float x0,y0,u,v,EC;
	long adr,adr0,AdresseImgBloc;
	ImageRGBA imgBloc;
	
	imgBloc.largeur=LargeurRecherche*2+1;
	imgBloc.hauteur=LargeurRecherche*2+1;
	imgBloc.data=new unsigned char[imgBloc.largeur*imgBloc.hauteur*4];
	adr0=(xb+yb*ImageTest.largeur)*4;
	imgBloc.x0=ImageTest.x0-xb;
	imgBloc.y0=ImageTest.y0-yb;
	
	MeilleurDeltaX=0;
	MeilleurDeltaY=0;
	EcartMin=1000*1000*1000;
	for (DeltaX=DecalageMinX;DeltaX<=DecalageMaxX;DeltaX+=DecalagePas)
	{
		for (DeltaY=DecalageMinY;DeltaY<=DecalageMaxY;DeltaY+=DecalagePas)
		{
			//Cration d'une nouvelle image extraite de ImageTest
			//avec interpolation bilinaire pour pouvoir faire des dplacements infrieurs au pixel
			AdresseImgBloc=0;
			for (y=0;y<imgBloc.hauteur;y++)
			{
				y0=y-DeltaY;
				y0i=y0;
				v=y0-y0i;
				for (x=0;x<imgBloc.largeur;x++)
				{
					x0=x-DeltaX;
					x0i=x0;
					u=x0-x0i;
					adr=adr0+(x0i+y0i*ImageTest.largeur)*4;
					if ((x0i+xb>=0) && (x0i+xb<ImageTest.largeur-1) && (y0i+yb>=0) && (y0i+yb<ImageTest.hauteur-1))
					{
						imgBloc.data[AdresseImgBloc]=ImageTest.data[adr]*(1-u)*(1-v)+ImageTest.data[adr+4]*u*(1-v)+ImageTest.data[adr+ImageTest.largeur*4]*(1-u)*v+ImageTest.data[adr+4+ImageTest.largeur*4]*u*v;
						imgBloc.data[AdresseImgBloc+1]=ImageTest.data[adr+1]*(1-u)*(1-v)+ImageTest.data[adr+4+1]*u*(1-v)+ImageTest.data[adr+ImageTest.largeur*4+1]*(1-u)*v+ImageTest.data[adr+4+ImageTest.largeur*4+1]*u*v;
						imgBloc.data[AdresseImgBloc+2]=ImageTest.data[adr+2]*(1-u)*(1-v)+ImageTest.data[adr+4+2]*u*(1-v)+ImageTest.data[adr+ImageTest.largeur*4+2]*(1-u)*v+ImageTest.data[adr+4+ImageTest.largeur*4+2]*u*v;
						imgBloc.data[AdresseImgBloc+3]=ImageTest.data[adr+3]*(1-u)*(1-v)+ImageTest.data[adr+4+3]*u*(1-v)+ImageTest.data[adr+ImageTest.largeur*4+3]*(1-u)*v+ImageTest.data[adr+4+ImageTest.largeur*4+3]*u*v;
					}
					else if ((x0i+xb>=0) && (x0i+xb<ImageTest.largeur) && (y0i+yb) && (y0i+yb<ImageTest.hauteur))
					{
					
						imgBloc.data[AdresseImgBloc]=ImageTest.data[adr];
						imgBloc.data[AdresseImgBloc+1]=ImageTest.data[adr+1];
						imgBloc.data[AdresseImgBloc+2]=ImageTest.data[adr+2];
						imgBloc.data[AdresseImgBloc+3]=ImageTest.data[adr+3];
					}
					else
					{
						imgBloc.data[AdresseImgBloc+3]=0;
					}
					AdresseImgBloc+=4;
				}
			}
			//Calcul de l'cart entre l'image gnre et l'image de rfrence
			EC=EcartImagesRGBA(ImageReference,imgBloc);
			if (EC<EcartMin)
			{
				EcartMin=EC;
				MeilleurDeltaX=DeltaX;
				MeilleurDeltaY=DeltaY;
			}
		}
	}
	delete [] imgBloc.data;
	
	return EcartMin;
}

//
//Calcul du meilleur dcalage pour un bloc donn
//Cas de dcalage en int
//MeilleurDeltaX et MeilleurDeltaY valeurs pour le retour du meilleur dcalage
//xb,yb coordonnes du centre du bloc  ajuster
//La fonction retourne la valeur de l'erreur commise
float MeilleurDecalageEntierComplet(float &MeilleurDeltaX,float &MeilleurDeltaY,int xb,int yb)
{
	int DeltaX,DeltaY;
	float EcartMin;
	int x,y,x0,y0;
	float EC;
	long adr,adr0,AdresseImgBloc;
	ImageRGBA imgBloc;
	
	imgBloc.largeur=LargeurRecherche*2+1;
	imgBloc.hauteur=LargeurRecherche*2+1;
	imgBloc.data=new unsigned char[imgBloc.largeur*imgBloc.hauteur*4];

	adr0=(xb+yb*ImageTest.largeur)*4;
	imgBloc.x0=ImageTest.x0-xb+LargeurRecherche;
	imgBloc.y0=ImageTest.y0-yb+LargeurRecherche;
	//Cration d'une nouvelle image extraite de ImageTest
	AdresseImgBloc=0;
	for (y=0;y<imgBloc.hauteur;y++)
	{
		y0=y-LargeurRecherche;
		for (x=0;x<imgBloc.largeur;x++)
		{
			x0=x-LargeurRecherche;
			adr=adr0+(x0+y0*ImageTest.largeur)*4;
			if ((x0>=0) && (x0<ImageTest.largeur) && (y0>=0) && (y0<ImageTest.hauteur))
			{
				imgBloc.data[AdresseImgBloc]=ImageTest.data[adr];
				imgBloc.data[AdresseImgBloc+1]=ImageTest.data[adr+1];
				imgBloc.data[AdresseImgBloc+2]=ImageTest.data[adr+2];
				imgBloc.data[AdresseImgBloc+3]=ImageTest.data[adr+3];
			}
			else
			{
				imgBloc.data[AdresseImgBloc+3]=0;
			}
			AdresseImgBloc+=4;
		}
	}
	
	MeilleurDeltaX=0;
	MeilleurDeltaY=0;
	EcartMin=1000*1000*1000;
	x0=imgBloc.x0;
	y0=imgBloc.y0;
	for (DeltaX=DecalageMinX;DeltaX<=DecalageMaxX;DeltaX+=DecalagePas)
	{
		for (DeltaY=DecalageMinY;DeltaY<=DecalageMaxY;DeltaY+=DecalagePas)
		{
			imgBloc.x0=x0-DeltaX;
			imgBloc.y0=y0-DeltaY;
			
			//Calcul de l'cart entre l'image gnre et l'image de rfrence
			EC=EcartImagesRGBA(ImageReference,imgBloc);
			if (EC<EcartMin)
			{
				EcartMin=EC;
				MeilleurDeltaX=DeltaX;
				MeilleurDeltaY=DeltaY;
			}
		}
	}
	delete [] imgBloc.data;
		
	return EcartMin;
}

//
//Calcul du meilleur dcalage pour un bloc donn
//Cas de dcalage en int
//MeilleurDeltaX et MeilleurDeltaY valeurs pour le retour du meilleur dcalage
//A l'appel de la fonction on doit y passer le dcalage le plus probable (celui calcul au bloc prcdent ou (0,0) s'il est inconnu).
//xb,yb coordonnes du centre du bloc  ajuster
//La fonction retourne la valeur de l'erreur commise
float MeilleurDecalageEntierStochastique(float &MeilleurDeltaX,float &MeilleurDeltaY,int xb,int yb)
{
	int DeltaX,DeltaY;
	float EcartMin;
	int x,y,x0,y0,k;
	long a;
	float EC;
	long adr,adr0,AdresseImgBloc;
	ImageRGBA imgBloc;
	int DeltaTotalX,DeltaTotalY;
		
	imgBloc.largeur=LargeurRecherche*2+1;
	imgBloc.hauteur=LargeurRecherche*2+1;
	imgBloc.data=new unsigned char[imgBloc.largeur*imgBloc.hauteur*4];

	adr0=(xb+yb*ImageTest.largeur)*4;
	imgBloc.x0=ImageTest.x0-xb;
	imgBloc.y0=ImageTest.y0-yb;
	
	EcartMin=1000*1000*1000;

	imgBloc.x0=ImageTest.x0-xb+LargeurRecherche;
	imgBloc.y0=ImageTest.y0-yb+LargeurRecherche;
	//Cration d'une nouvelle image extraite de ImageTest
	AdresseImgBloc=0;
	for (y=0;y<imgBloc.hauteur;y++)
	{
		y0=y-LargeurRecherche;
		for (x=0;x<imgBloc.largeur;x++)
		{
			x0=x-LargeurRecherche;
			adr=adr0+(x0+y0*ImageTest.largeur)*4;
			if ((x0>=0) && (x0<ImageTest.largeur) && (y0>=0) && (y0<ImageTest.hauteur))
			{
				imgBloc.data[AdresseImgBloc]=ImageTest.data[adr];
				imgBloc.data[AdresseImgBloc+1]=ImageTest.data[adr+1];
				imgBloc.data[AdresseImgBloc+2]=ImageTest.data[adr+2];
				imgBloc.data[AdresseImgBloc+3]=ImageTest.data[adr+3];
			}
			else
			{
				imgBloc.data[AdresseImgBloc+3]=0;
			}
			AdresseImgBloc+=4;
		}
	}

	//Initialisation du dcalage  partir de celui donn en entre
	DeltaTotalX=MeilleurDeltaX;
	DeltaTotalY=MeilleurDeltaY;
	
	x0=imgBloc.x0;
	y0=imgBloc.y0;
	k=0;
	while ((EcartMin>Epsilon/10) && (k<DecalageMaxX*DecalageMaxX))
	{
		k++;
		a=rand();
		if (a<RAND_MAX/3)
		{
			DeltaX=-1;
		}
		else
		{
			if (a<RAND_MAX*2/3)
			{
				DeltaX=0;
			}
			else DeltaX=1;
		}
		a=rand();
		if (a<RAND_MAX/3)
		{
			DeltaY=-1;
		}
		else
		{
			if (a<RAND_MAX*2/3)
			{
				DeltaY=0;
			}
			else DeltaY=1;
		}
		
		imgBloc.x0=x0-DeltaTotalX;
		imgBloc.y0=y0-DeltaTotalY;
		
		//Calcul de l'cart entre l'image gnre et l'image de rfrence
		EC=EcartImagesRGBA(ImageReference,imgBloc);
		if (EC<EcartMin)
		{
			EcartMin=EC;
			MeilleurDeltaX=DeltaX;
			MeilleurDeltaY=DeltaY;
		}
		else
		{
			if (rand()*1.0/RAND_MAX<exp(-(EC-EcartMin)*log(k*1.0)))
			{				
				EcartMin=EC;
				MeilleurDeltaX=DeltaX;
				MeilleurDeltaY=DeltaY;
			}
			else
			{
				MeilleurDeltaX=0;
				MeilleurDeltaY=0;
			}
		}
		DeltaTotalX+=MeilleurDeltaX;
		DeltaTotalY+=MeilleurDeltaY;
	}
	if (EcartMin>Epsilon)
	{
		_nbtotal++;
		_nbcomplet++;
		printf("Complet");
		EcartMin=MeilleurDecalageEntierComplet(MeilleurDeltaX,MeilleurDeltaY,xb,yb);	
		if (EcartMin<Epsilon)
		{
			printf("++\n");
			_nbcompletplus++;
		}
		else
		{
			printf("--\n");
			_nbcompletmoins++;
		}
	}
	else 
	{
		_nbtotal++;
		printf("OK(%i)",k);
		MeilleurDeltaX=DeltaTotalX;
		MeilleurDeltaY=DeltaTotalY;
	}
	
	delete [] imgBloc.data;
	
	return EcartMin;
}


//
//Ajustage de l'image par blocs (recherche et dformation)
//
void AjusteBlocs(void)
{
	int xb,yb,xbG,xbD,ybH,ybB,i,j,pourcentage,pourcentageAffiche;
	float MeilleurDeltaX,MeilleurDeltaY,EcartMin;
	int xBloc,yBloc;
	int dpi;
	ImageRGBA imgRes;
	TransfoBlocs TB;
	int sb;
	long dmin;
	
	
	ImageRGBA imgTest0,imgRef0;
	
	//Sauvergarde de l'image de test et de l'image de ref en mmoire
	//le calcul va se faire sur leurs drives (option a prevoir)

	imgTest0=CopieImageRGBA(ImageTest);
	imgRef0=CopieImageRGBA(ImageReference);
	DeriveImageRGBA(ImageTest);
	DeriveImageRGBA(ImageReference);
	
	
	
		
	printf("Ajustage par blocs\n");
	printf("Calcul du meilleur ajustage");
	//Calcul du nombre de blocs
	TB.nbBlocsX=0;
	TB.nbBlocsY=0;
	for (xb=LargeurRecherche+1-DecalageMinX;xb<ImageTest.largeur-LargeurRecherche-1-DecalageMaxX;xb+=LargeurGrille)
	{
		TB.nbBlocsX++;
	}
	for (yb=LargeurRecherche+1-DecalageMinY;yb<ImageTest.hauteur-LargeurRecherche-1-DecalageMaxY;yb+=LargeurGrille)
	{
		TB.nbBlocsY++;
	}
	
	TB.deltaX=new float[(TB.nbBlocsX+1)*(TB.nbBlocsY+1)];
	TB.deltaY=new float[(TB.nbBlocsX+1)*(TB.nbBlocsY+1)];
	TB.Ecart=new float[(TB.nbBlocsX+1)*(TB.nbBlocsY+1)];
	sb=TB.nbBlocsX;
	
	printf(" (%i blocs en largeur %i blocs en hauteur)\n",TB.nbBlocsX,TB.nbBlocsY);	
	printf("--------------------------------------------------\n");
	pourcentageAffiche=0;
	xBloc=0;
	
	for (xb=LargeurRecherche+1-DecalageMinX;xb<ImageTest.largeur-LargeurRecherche-1-DecalageMaxX;xb+=LargeurGrille)
	{
		//Affichage de la progression du calcul
		pourcentage=xBloc*50/TB.nbBlocsX;
		pourcentage=pourcentage-pourcentageAffiche+1;
		for (i=0;i<pourcentage;i++)
		{
			printf("*");
			pourcentageAffiche++;
		}
		fflush(stdout);
		
		yBloc=0;
		MeilleurDeltaX=0;
		MeilleurDeltaY=0;
		for (yb=LargeurRecherche+1-DecalageMinY;yb<ImageTest.hauteur-LargeurRecherche-1-DecalageMaxY;yb+=LargeurGrille)
		{
			//printf("X=%i Y=%i",xBloc,yBloc);
			//(xb,yb) centre de bloc  ajuster
			
		
			//Avant de passer  la suite on vrifie s'il peut y avoir superposition entre le futur bloc
			// et l'image de rfrence. Si ce n'est pas le cas, on ne fait pas de calucls pour rien
			// utile surtout pour acclrer le calcul des mosaiques
			//Calcul des coordonnes des 4 coins avec leur dplacement maxi autoris
			//coord par rapport  l'image de rfrence
			xbG=xb-LargeurRecherche+DecalageMinX-ImageTest.x0+ImageReference.x0;
			xbD=xb+LargeurRecherche+DecalageMaxX-ImageTest.x0+ImageReference.x0;
			ybH=yb-LargeurRecherche+DecalageMinY-ImageTest.y0+ImageReference.y0;
			ybB=yb+LargeurRecherche+DecalageMaxX-ImageTest.y0+ImageReference.y0;
			if ((xbG>=0) && (xbD<=ImageReference.largeur) && (ybB>=0) && (ybH<=ImageReference.hauteur))
			{
				//Recherche du meilleur dcalage possible pour la bloc considr
				//2fonctions diffrentes selon qu'on cherche un dcalage entier ou un dcalage fractionnaire
				dpi=DecalagePas;
				if (DecalagePas!=dpi) EcartMin=MeilleurDecalage(MeilleurDeltaX,MeilleurDeltaY,xb,yb);
				else EcartMin=MeilleurDecalageEntierComplet/*Stochastique*/(MeilleurDeltaX,MeilleurDeltaY,xb,yb);
			}
			else
			{
				//Cas ou il n'y avait pas de superposition entre les blocs
				//on peut amliorer en reprenant le dcalage des blocs voisins  la fin
				MeilleurDeltaX=0;
				MeilleurDeltaY=0;
				EcartMin=-10;	//valeur ngative signifie qu'il n'y a pas eu superposition
			}
			
			
			//Mise en mmoire du meilleur dcalage pour le bloc examin
			TB.deltaX[xBloc+yBloc*sb]=MeilleurDeltaX;
			TB.deltaY[xBloc+yBloc*sb]=MeilleurDeltaY;
			TB.Ecart[xBloc+yBloc*sb]=EcartMin;
			//printf("  DeltaX=%f DeltaY=%f  Ecart=%f\n",MeilleurDeltaX,MeilleurDeltaY,EcartMin);
			yBloc++;
		}
		xBloc++;	
	}
	
	printf("\n");
	//On complte les dcalages pour les blocs qui ne se superposaient pas
	//reprise des dcalages des blocs voisins
	dmin=32000*32000;
	for (yb=0;yb<TB.nbBlocsY;yb++)
	{
		for (xb=0;xb<TB.nbBlocsX;xb++)
		{
			if (TB.Ecart[xb+yb*sb]<0)
			{
				//Recalculer deltaX et deltaY
				for (i=0;i<TB.nbBlocsX;i++)
					for (j=0;j<TB.nbBlocsY;j++)
						if ((TB.Ecart[i+j*sb]>0) && ( (xb-i)*(xb-i)+(yb-j)*(yb-j)<dmin) )
						{
							dmin=(xb-i)*(xb-i)+(yb-j)*(yb-j);
							TB.Ecart[xb+yb*sb]=TB.Ecart[i+j*sb];
						}
			}
		}

	}


	//Lissage de la deformation (rajouter une option pour l'activer)
	
	
	//Recomposition de l'image en tenant compte des dcalages
	printf("Deformation de l'image\n");
	
	/*printf("DeltaX\n");
	for (yb=0;yb<TB.nbBlocsY;yb++)
	{
		for (xb=0;xb<TB.nbBlocsX;xb++)
		{
			i=TB.deltaX[xb+yb*sb];
			if (i>=0) printf("+%i",i);
			else printf("%i",i);	
		}
		printf("\n");
	}
	printf("\n");
	printf("DeltaY\n");
	for (yb=0;yb<TB.nbBlocsY;yb++)
	{
		for (xb=0;xb<TB.nbBlocsX;xb++)
		{
			j=TB.deltaY[xb+yb*sb];
			if (j>=0) printf("+%i",j);
			else printf("%i",j);	
		}
		printf("\n");
	}
	printf("\n");
	printf("total: %i  complet : %i   plus : %i   moins : %i\n",_nbtotal,_nbcomplet,_nbcompletplus,_nbcompletmoins);
    */

	//Retour aux images de depart pour effectuer la transformation
	
	delete [] ImageTest.data;
	delete [] ImageReference.data;
	ImageTest=imgTest0;
	ImageReference=imgRef0;


	imgRes=TransfoBlocsImageRGBA(ImageTest,TB,LargeurGrille,LargeurGrille);
	delete [] ImageTest.data;
	delete [] TB.deltaX;
	delete [] TB.deltaY;
	delete [] TB.Ecart;

	ImageTest=imgRes;
}

//
//Transformation selon une grille
//
ImageRGBA TransfoBlocsImageRGBA(ImageRGBA imgOrigine,TransfoBlocs &TB,int lg,int hg)
{
	int x,y;
	float u,v,n,m;
	int XBloc1,YBloc1,XBloc2,YBloc2;
	float DeltaX,DeltaY;
	float x0,y0;
	int x0i,y0i;
	int sb=TB.nbBlocsX;
	long adr0,adr;
	float alpha,beta,gamma,delta;
	
	ImageRGBA imgRes;
	
	imgRes.largeur=imgOrigine.largeur;
	imgRes.hauteur=imgOrigine.hauteur;
	imgRes.data=new unsigned char[imgRes.largeur*imgRes.hauteur*4];
	imgRes.x0=imgOrigine.x0;	//A verifier...
	imgRes.y0=imgOrigine.y0;
	
	adr=0;
	for (y=0;y<imgRes.hauteur;y++)
	{
		for (x=0;x<imgRes.largeur;x++)
		{
			//Recherche des 4 centres de blocs qui encadre le point(x;y)
			// seulement 1 dans les coins: Ce centre est le seul qui ait de l'effet
			// seulement 2 sur les bords: On ne tient compte que de ces deux
			// si on est dans la partie totalement  droite ou en bas de l'image, il se peut que le pixel ne fasse partie d'aucun bloc
			// dans ce cas on prend le centre de bloc immdiatement  gauche et au dessus, sans faire d'interpolation
			
			//Premier centre de bloc qui soit situ au dessus et  gauche de (x;y)
			if (x>lg/2)
			{
				XBloc1=(x-lg/2)/lg;
				u=(x-(XBloc1*lg*1.0+lg*0.5))/lg;
				
				//Cas d'un pixel qui est au del du dernier bloc)
				if (XBloc1>=TB.nbBlocsX)
				{
					XBloc1=TB.nbBlocsX-1;
					u=0;
				}
			}
			else
			{
				XBloc1=0;
				u=0;
			}
			if (y>hg/2)
			{
				YBloc1=(y-hg/2)/hg;
				v=(y-(YBloc1*hg*1.0+hg*0.5))/hg;
				
				//Cas d'un pixel en dessous du dernier bloc
				if (YBloc1>=TB.nbBlocsY)
				{
					YBloc1=TB.nbBlocsY-1;
					v=0;
				}
			}
			else
			{
				YBloc1=0;
				v=0;
			}

			u=1-u;
			v=1-v;
			
			//Calcul des blocs suivants pour l'interpolation linaire (rptition du mme bloc dans le cas du bord de l'image)
			XBloc2=XBloc1+1;
			if (XBloc2>=TB.nbBlocsX) XBloc2=XBloc1;
			YBloc2=YBloc1+1;
			if (YBloc2>=TB.nbBlocsY) YBloc2=YBloc1;
						
			//Calcul de xDepart et yDepart par interpolation bilinaire				
			DeltaX=(TB.deltaX[XBloc1+YBloc1*sb]*u*v+TB.deltaX[XBloc2+YBloc1*sb]*(1-u)*v+TB.deltaX[XBloc1+YBloc2*sb]*u*(1-v)+TB.deltaX[XBloc2+YBloc2*sb]*(1-u)*(1-v));
			DeltaY=(TB.deltaY[XBloc1+YBloc1*sb]*u*v+TB.deltaY[XBloc2+YBloc1*sb]*(1-u)*v+TB.deltaY[XBloc1+YBloc2*sb]*u*(1-v)+TB.deltaY[XBloc2+YBloc2*sb]*(1-u)*(1-v));
			//EcartMin=(TB.Ecart[XBloc1+YBloc1*sb]*u*v+TB.Ecart[XBloc2+YBloc1*sb]*(1-u)*v+TB.Ecart[XBloc1+YBloc2*sb]*u*(1-v)+TB.Ecart[XBloc2+YBloc2*sb]*(1-u)*(1-v));
						
			x0=x-DeltaX;
			y0=y-DeltaY;
			//Troncature pour avoir des entiers. m et n sont utiliss pour une interpolation bilinaire
			x0i=x0;
			y0i=y0;
			m=x0-x0i;
			n=y0-y0i;
			
			//La dernire ligne et la dernire colonne ne peuvent pas tre calcules par interpolation linaire.
			if ((x0i>=0) && (x0i<imgOrigine.largeur-1) && (y0i>=0) && (y0i<imgOrigine.hauteur-1))
			{
				adr0=(x0i+y0i*imgOrigine.largeur)*4;
				alpha=(1-m)*(1-n);
				beta=m*(1-n);
				gamma=(1-m)*n;
				delta=m*n;
				imgRes.data[adr+0]=alpha*imgOrigine.data[adr0+0]+beta*imgOrigine.data[adr0+0+4]+gamma*imgOrigine.data[adr0+0+imgOrigine.largeur*4]+delta*imgOrigine.data[adr0+0+imgOrigine.largeur*4+4];
				imgRes.data[adr+1]=alpha*imgOrigine.data[adr0+1]+beta*imgOrigine.data[adr0+1+4]+gamma*imgOrigine.data[adr0+1+imgOrigine.largeur*4]+delta*imgOrigine.data[adr0+1+imgOrigine.largeur*4+4];
				imgRes.data[adr+2]=alpha*imgOrigine.data[adr0+2]+beta*imgOrigine.data[adr0+2+4]+gamma*imgOrigine.data[adr0+2+imgOrigine.largeur*4]+delta*imgOrigine.data[adr0+2+imgOrigine.largeur*4+4];
				imgRes.data[adr+3]=alpha*imgOrigine.data[adr0+3]+beta*imgOrigine.data[adr0+3+4]+gamma*imgOrigine.data[adr0+3+imgOrigine.largeur*4]+delta*imgOrigine.data[adr0+3+imgOrigine.largeur*4+4];
				//imgRes.data[adr+1]=EcartMin*255/Epsilon;
			}
			else if ((x0i>=0) && (x0i<imgOrigine.largeur) && (y0i>=0) && (y0i<imgOrigine.hauteur))
			{
				adr0=(x0i+y0i*ImageTest.largeur)*4;
				
				imgRes.data[adr+0]=imgOrigine.data[adr0+0];
				imgRes.data[adr+1]=imgOrigine.data[adr0+1];
				imgRes.data[adr+2]=imgOrigine.data[adr0+2];
				imgRes.data[adr+3]=imgOrigine.data[adr0+3];
			}
			else
			{
				imgRes.data[adr+3]=0;				
			}
			adr+=4;
		}
	}	
	return imgRes;
}

//
//Transformation affine avec interpolation bilinaire
//
ImageRGBA TransfoAffineImageRGBA(ImageRGBA imgOrigine,float Theta,float DeltaX,float DeltaY)
{
	//Pour l'instant la nouvelle image conserve la taille de l'ancienne
	ImageRGBA img;
	int x,y;
	float x0,y0;
	float xt[4],yt[4];	//Image des coins
	float u,v;
	int x0i,y0i;
	double c=cos(Theta);
	double s=sin(Theta);
	unsigned long adr,adr0;
	
		
	//Calcul des nouvelles dimensions de l'image
	//Transformation des 4 coins (transfo inverse de la suivante
	xt[0]=c*(0-imgOrigine.x0)+s*(0-imgOrigine.y0)-DeltaX;
	yt[0]=-s*(0-imgOrigine.x0)+c*(0-imgOrigine.y0)-DeltaY;
	xt[1]=c*(imgOrigine.largeur-imgOrigine.x0)+s*(0-imgOrigine.y0)-DeltaX;
	yt[1]=-s*(imgOrigine.largeur-imgOrigine.x0)+c*(0-imgOrigine.y0)-DeltaY;
	xt[2]=c*(0-imgOrigine.x0)+s*(imgOrigine.hauteur-imgOrigine.y0)-DeltaX;
	yt[2]=-s*(0-imgOrigine.x0)+c*(imgOrigine.hauteur-imgOrigine.y0)-DeltaY;
	xt[3]=c*(imgOrigine.largeur-imgOrigine.x0)+s*(imgOrigine.hauteur-imgOrigine.y0)-DeltaX;
	yt[3]=-s*(imgOrigine.largeur-imgOrigine.x0)+c*(imgOrigine.hauteur-imgOrigine.y0)-DeltaY;
	
	img.largeur=max(max(xt[0],xt[1]),max(xt[2],xt[3]))-min(min(xt[0],xt[1]),min(xt[2],xt[3]));
	img.hauteur=max(max(yt[0],yt[1]),max(yt[2],yt[3]))-min(min(yt[0],yt[1]),min(yt[2],yt[3]));
	//Point de ref sur la nouvelle image
	if (Theta>0)
	{
		x0=0;
		y0=imgOrigine.largeur*s;
	}
	else
	{
		x0=-imgOrigine.hauteur*s;
		y0=0;
	}
	img.x0=x0+c*imgOrigine.x0+s*imgOrigine.y0;
	img.y0=y0-s*imgOrigine.x0+c*imgOrigine.y0;
	
	//Reservation de place
	img.data=new unsigned char[img.largeur*img.hauteur*4];
	//printf("Largeur=%i Hauteur=%i",img.largeur,img.hauteur);
	//Rotation et translation
	adr=0;
	for (y=0;y<img.hauteur;y++)
	{
		for (x=0;x<img.largeur;x++)
		{
			//calcul de (x0,y0) image de (x,y) par la rotation d'angle -Theta puis par translation de (-DeltaX,-DeltaY)
			x0=c*(x-img.x0)-s*(y-img.y0)+imgOrigine.x0+DeltaX;
			y0=s*(x-img.x0)+c*(y-img.y0)+imgOrigine.y0+DeltaY;
			
			//Copie des pixels
			x0i=x0;
			u=x0-x0i;
			y0i=y0;
			v=y0-y0i;
			if ((x0i>=0) && (x0i<imgOrigine.largeur-1) && (y0i>=0) && (y0i<imgOrigine.hauteur-1))
			{
				adr0=(x0i+y0i*imgOrigine.largeur)*4;
				img.data[adr]=imgOrigine.data[adr0]*(1-u)*(1-v)+imgOrigine.data[adr0+4]*u*(1-v)+imgOrigine.data[adr0+imgOrigine.largeur*4]*(1-u)*v+imgOrigine.data[adr0+(1+imgOrigine.largeur)*4]*u*v;
				img.data[adr+1]=imgOrigine.data[adr0+1]*(1-u)*(1-v)+imgOrigine.data[adr0+4+1]*u*(1-v)+imgOrigine.data[adr0+imgOrigine.largeur*4+1]*(1-u)*v+imgOrigine.data[adr0+(1+imgOrigine.largeur)*4+1]*u*v;
				img.data[adr+2]=imgOrigine.data[adr0+2]*(1-u)*(1-v)+imgOrigine.data[adr0+4+2]*u*(1-v)+imgOrigine.data[adr0+imgOrigine.largeur*4+2]*(1-u)*v+imgOrigine.data[adr0+(1+imgOrigine.largeur)*4+2]*u*v;
				img.data[adr+3]=imgOrigine.data[adr0+3]*(1-u)*(1-v)+imgOrigine.data[adr0+4+3]*u*(1-v)+imgOrigine.data[adr0+imgOrigine.largeur*4+3]*(1-u)*v+imgOrigine.data[adr0+(1+imgOrigine.largeur)*4+3]*u*v;
			}
			else
			{
				if ((x0i>=0) && (x0i<imgOrigine.largeur) && (y0i>=0) && (y0i<imgOrigine.hauteur))
				{
					adr0=(x0i+y0i*imgOrigine.largeur)*4;
					img.data[adr]=imgOrigine.data[adr0];
					img.data[adr+1]=imgOrigine.data[adr0+1];
					img.data[adr+2]=imgOrigine.data[adr0+2];
					img.data[adr+3]=imgOrigine.data[adr0+3];			
				}
				else
				{
					img.data[adr]=0;			
					img.data[adr+1]=0;			
					img.data[adr+2]=0;			
					img.data[adr+3]=0;
				}
			}
			adr=adr+4;
		}
	}
	return img;	
}

//
//Copie d'une image
//
ImageRGBA CopieImageRGBA(ImageRGBA & img)
{
	ImageRGBA img2;
	long x;

	img2.largeur=img.largeur;
	img2.hauteur=img.hauteur;
	img2.x0=img.x0;
	img2.y0=img.y0;
	img2.data= new unsigned char[img2.largeur*img2.hauteur*4];

	//Copie
	for (x=0;x<img.largeur*img.hauteur*4;x++)
	{
		img2.data[x]=img.data[x];
	}
	return img2;
}

//
//Redimensionnement d'une image
//zoom<1 -> reduction
//
ImageRGBA ZoomImageRGBA(ImageRGBA & img,float zoom)
{
	ImageRGBA img2;
	int x,y,x0i,y0i;
	float x0,y0,u,v;
	long adr,adr0;

	img2.largeur=img.largeur*zoom;
	img2.hauteur=img.hauteur*zoom;
	img2.x0=img.x0*zoom;
	img2.y0=img.y0*zoom;
	img2.data= new unsigned char[img2.largeur*img2.hauteur*4];

	//Remplissage de l'image 2
	for (x=0;x<img2.largeur;x++)
		for (y=0;y<img2.hauteur;y++)
		{
			x0=x/zoom;
			y0=y/zoom;
			//Copie des pixels
			x0i=x0;
			u=x0-x0i;
			y0i=y0;
			v=y0-y0i;
			if ((x0i>=0) && (x0i<img.largeur-1) && (y0i>=0) && (y0i<img.hauteur-1))
			{
				adr=(x+y*img2.largeur)*4;
				adr0=(x0i+y0i*img.largeur)*4;
				img2.data[adr]=img.data[adr0]*(1-u)*(1-v)+img.data[adr0+4]*u*(1-v)+img.data[adr0+img.largeur*4]*(1-u)*v+img.data[adr0+(1+img.largeur)*4]*u*v;
				img2.data[adr+1]=img.data[adr0+1]*(1-u)*(1-v)+img.data[adr0+4+1]*u*(1-v)+img.data[adr0+img.largeur*4+1]*(1-u)*v+img.data[adr0+(1+img.largeur)*4+1]*u*v;
				img2.data[adr+2]=img.data[adr0+2]*(1-u)*(1-v)+img.data[adr0+4+2]*u*(1-v)+img.data[adr0+img.largeur*4+2]*(1-u)*v+img.data[adr0+(1+img.largeur)*4+2]*u*v;
				img2.data[adr+3]=img.data[adr0+3]*(1-u)*(1-v)+img.data[adr0+4+3]*u*(1-v)+img.data[adr0+img.largeur*4+3]*(1-u)*v+img.data[adr0+(1+img.largeur)*4+3]*u*v;
			}
			else
			{
				if ((x0i>=0) && (x0i<img.largeur) && (y0i>=0) && (y0i<img.hauteur))
				{
					adr=(x+y*img2.largeur)*4;
					adr0=(x0i+y0i*img.largeur)*4;
					img2.data[adr]=img.data[adr0];
					img2.data[adr+1]=img.data[adr0+1];
					img2.data[adr+2]=img.data[adr0+2];
					img2.data[adr+3]=img.data[adr0+3];			
				}
				else
				{
					adr=(x+y*img2.largeur)*4;
					img.data[adr]=0;			
					img.data[adr+1]=0;			
					img.data[adr+2]=0;			
					img.data[adr+3]=0;
				}
			}
		}
		
	return img2;
}

//
//Translation d'une image
//la translation est effectivement effectue
//
ImageRGBA TranslationImageRGBA(ImageRGBA & img,float DeltaX,float DeltaY)
{
	ImageRGBA img2;
	int x,y,x0i,y0i;
	float x0,y0,u,v;
	long adr,adr0;

	img2.largeur=img.largeur;
	img2.hauteur=img.hauteur;
	img2.x0=img.x0;
	img2.y0=img.y0;
	img2.data= new unsigned char[img2.largeur*img2.hauteur*4];

	//Remplissage de l'image 2
	for (x=0;x<img2.largeur;x++)
		for (y=0;y<img2.hauteur;y++)
		{
			x0=x-DeltaX;
			y0=y-DeltaY;
			//Copie des pixels
			x0i=x0;
			u=x0-x0i;
			y0i=y0;
			v=y0-y0i;
			if ((x0i>=0) && (x0i<img.largeur-1) && (y0i>=0) && (y0i<img.hauteur-1))
			{
				adr=(x+y*img2.largeur)*4;
				adr0=(x0i+y0i*img.largeur)*4;
				img2.data[adr]=img.data[adr0]*(1-u)*(1-v)+img.data[adr0+4]*u*(1-v)+img.data[adr0+img.largeur*4]*(1-u)*v+img.data[adr0+(1+img.largeur)*4]*u*v;
				img2.data[adr+1]=img.data[adr0+1]*(1-u)*(1-v)+img.data[adr0+4+1]*u*(1-v)+img.data[adr0+img.largeur*4+1]*(1-u)*v+img.data[adr0+(1+img.largeur)*4+1]*u*v;
				img2.data[adr+2]=img.data[adr0+2]*(1-u)*(1-v)+img.data[adr0+4+2]*u*(1-v)+img.data[adr0+img.largeur*4+2]*(1-u)*v+img.data[adr0+(1+img.largeur)*4+2]*u*v;
				img2.data[adr+3]=img.data[adr0+3]*(1-u)*(1-v)+img.data[adr0+4+3]*u*(1-v)+img.data[adr0+img.largeur*4+3]*(1-u)*v+img.data[adr0+(1+img.largeur)*4+3]*u*v;
			}
			else
			{
				if ((x0i>=0) && (x0i<img.largeur) && (y0i>=0) && (y0i<img.hauteur))
				{
					adr=(x+y*img2.largeur)*4;
					adr0=(x0i+y0i*img.largeur)*4;
					img2.data[adr]=img.data[adr0];
					img2.data[adr+1]=img.data[adr0+1];
					img2.data[adr+2]=img.data[adr0+2];
					img2.data[adr+3]=img.data[adr0+3];			
				}
				else
				{
					adr=(x+y*img2.largeur)*4;
					img.data[adr]=0;			
					img.data[adr+1]=0;			
					img.data[adr+2]=0;			
					img.data[adr+3]=0;
				}
			}
		}
		
	return img2;
}

//
//Filtrage par Symetric Nearest Neighbor
//
ImageRGBA FiltreSNN(ImageRGBA & img0,int rayon)
{
	ImageRGBA img;
	long x,y;
	int i,j;
	long r,v,b,valeurCentre,m,n;
	
	img.largeur=img0.largeur;
	img.hauteur=img0.hauteur;
	img.data=new unsigned char[img.largeur*img.hauteur*4];

	for (x=rayon;x<img.largeur-rayon;x++)
	{
		for (y=rayon;y<img.hauteur-rayon;y++)
		{
			r=0;
			v=0;
			b=0;
			//
			for (i=0;i<rayon+1;i++)
			{
				for (j=-rayon;j<rayon+1;j++)
				{
					valeurCentre=img0.data[(x+y*img.largeur)*4]+img0.data[(x+y*img.largeur)*4+1]+img0.data[(x+y*img.largeur)*4+2];
					m=img0.data[(x+i+(y+j)*img.largeur)*4]+img0.data[(x+i+(y+j)*img.largeur)*4+1]+img0.data[((x+i)+(y+j)*img.largeur)*4+2];
					n=img0.data[(x-i+(y-j)*img.largeur)*4]+img0.data[(x-i+(y-j)*img.largeur)*4+1]+img0.data[((x-i)+(y-j)*img.largeur)*4+2];
					if ( abs(valeurCentre-m)<abs(valeurCentre-n) )
					{
						r=r+img0.data[(x+i+(y+j)*img.largeur)*4];
						v=v+img0.data[(x+i+(y+j)*img.largeur)*4+1];
						b=b+img0.data[(x+i+(y+j)*img.largeur)*4+2];
					}
					else
					{
						r=r+img0.data[(x-i+(y-j)*img.largeur)*4];
						v=v+img0.data[(x-i+(y-j)*img.largeur)*4+1];
						b=b+img0.data[(x-i+(y-j)*img.largeur)*4+2];
					}
				}
			}
			//
			img.data[(x+y*img.largeur)*4]=r/((rayon+1)*(2*rayon+1));
			img.data[(x+y*img.largeur)*4+1]=v/((rayon+1)*(2*rayon+1));
			img.data[(x+y*img.largeur)*4+2]=b/((rayon+1)*(2*rayon+1));

		}
	}
	
	return img;
}

//
//Extrait une sous image de largeur donn et la fait tourner autour de son point (x0,y0)
//
ImageRGBA ExtraitImageRotationRGBA(const ImageRGBA & ImageOrigine,int LargeurExtrait,int HauteurExtrait,float Theta)
{
	ImageRGBA img;
	int x,y;
	int x0i,y0i;
	long ad1,ad2,fin;
	float c=cos(Theta);
	float s=sin(Theta);
	float a,b;
	
	img.largeur=LargeurExtrait;
	img.hauteur=HauteurExtrait;
	img.x0=LargeurExtrait/2;
	img.y0=HauteurExtrait/2;
	
	img.data=new unsigned char[img.largeur*img.hauteur*4];
	
	for (y=0;y<HauteurExtrait;y++)
	{
		ad1=y*img.largeur*4;
		//Attention : on ne gnre qu'un ligne sur 2 pour gagner du temps
		if (y%2==0)
		{		
			a=-s*(y-img.y0)+ImageOrigine.x0;
			b=c*(y-img.y0)+ImageOrigine.y0;
			for (x=0;x<LargeurExtrait;x++)
			{
				//calcul de (x0,y0) image de (x,y) par la rotation d'angle -Theta puis par translation de (-DeltaX,-DeltaY)
				x0i=c*(x-img.x0)+a;
				y0i=s*(x-img.x0)+b;
				
				//Copie des pixels
				if ((x0i>=0) && (x0i<ImageOrigine.largeur) && (y0i>=0) && (y0i<ImageOrigine.hauteur))
				{
					ad2=(x0i+y0i*ImageOrigine.largeur)*4;
					/*img.data[ad1]=ImageOrigine.data[ad2];
					img.data[ad1+1]=ImageOrigine.data[ad2+1];
					img.data[ad1+2]=ImageOrigine.data[ad2+2];
					img.data[ad1+3]=ImageOrigine.data[ad2+3];*/
					*((long *) (img.data+ad1))=*((long *) (ImageOrigine.data+ad2));
	 			}
				else img.data[ad1+3]=0;
				ad1+=4;
			}
		}
		else {fin=ad1+img.largeur*4;for (ad2=ad1;ad2<fin;ad2++) img.data[ad2+3]=0;}
	}
	return img;
}

//
//Calcul de la derivee d'une image
//
void DeriveImageRGBA(ImageRGBA & img)
{
	long x,y,k;
	unsigned char *finImage=img.data+img.largeur+img.hauteur*4;
	unsigned char *c;

	//Moyenne des composantes dans le canal rouge
	for (c=img.data;c<finImage;c+=4)
	{
		k=*c+*(c+1)+*(c+2);
		*c=k/3;
	}
	// d/dx dans le vert et d/dy dans le bleu
	for (y=1;y<img.hauteur-1;y++)
		for (x=1;x<img.largeur-1;x++)
		{
			img.data[(x+y*img.largeur)*4+1]=(img.data[(x-1+y*img.largeur)*4]-img.data[(x+1+y*img.largeur)*4])/2+128;
			img.data[(x+y*img.largeur)*4+2]=(img.data[(x+(y-1)*img.largeur)*4]-img.data[(x+(y+1)*img.largeur)*4])/2+128;
		}
	//SauveBMP(img,"Derivee.bmp",0,NULL);
}

//Retourne l'cart entre 2 images
//Ecart=Somme(ecart intensite des pixels au carr)/Nombre de pixels utiliss dans le calcul
//Les pixels transparents ne sont pas pris en compte
float EcartImagesRGBA(const ImageRGBA & img1,const ImageRGBA & img2)
{
	int x;
	long FinImg2=(long) img2.data+(img2.largeur*img2.hauteur)*4;
	long FinImg1=(long) img1.data+(img1.largeur*img1.hauteur)*4;
	long LargeurOctetsImg1=img1.largeur*4;
	long LargeurOctetsImg2=img2.largeur*4;
	long LargeurOctetsIntersection;
	unsigned char * DebutLigneImg1=img1.data;
	unsigned char * DebutLigneImg2=img2.data;
	unsigned char *RefPixel1;
	unsigned char *RefPixel2;
	long d1,d2,d3;
	float Difference;
	float e=0;
	long NbPixel=0;
	
	//Positionnement des pointeurs au dbut de la zone intersection des images
	if (img1.x0>=img2.x0)
	{
           DebutLigneImg1+=(img1.x0-img2.x0)*4;
		LargeurOctetsIntersection=min(img1.data-DebutLigneImg1+LargeurOctetsImg1,LargeurOctetsImg2);
	}
	else
	{
           DebutLigneImg2+=(img2.x0-img1.x0)*4;
		LargeurOctetsIntersection=min(img2.data-DebutLigneImg2+LargeurOctetsImg2,LargeurOctetsImg1);
	}
	
	if (img1.y0>=img2.y0) DebutLigneImg1+=(img1.y0-img2.y0)*LargeurOctetsImg1;
	else DebutLigneImg2+=(img2.y0-img1.y0)*LargeurOctetsImg2;
	

        float Epsilon_sur_2=Epsilon/2;
                            
	while (((long) DebutLigneImg2<FinImg2) && ((long) DebutLigneImg1<FinImg1))
	{
		RefPixel1=DebutLigneImg1;
		RefPixel2=DebutLigneImg2;
		
			for (x=0;x<LargeurOctetsIntersection;x+=4)
			{
				if ((*(RefPixel2+3)!=0) && (*(RefPixel1+3)!=0))
				{
					//Cas ou il ya effectivement superposition
					d1=(*(RefPixel2))-(*(RefPixel1));
					d2=(*(RefPixel2+1))-(*(RefPixel1+1));
					d3=(*(RefPixel2+2))-(*(RefPixel1+2));
					Difference=(d1*d1+4*(d2*d2+d3*d3))/*/(3*255)*/;
                                        e+=(Difference*Difference)/255;
					e+=Difference;
                                        ++NbPixel;
				}
				else
				{
					//Sinon on ajoute une valeur moyenne  l'erreur
					e+=Epsilon_sur_2;
					++NbPixel;
				}
				RefPixel1+=4;
				RefPixel2+=4;
			}
		
		DebutLigneImg1+=LargeurOctetsImg1;
		DebutLigneImg2+=LargeurOctetsImg2;
	} 
	//printf("%i ",NbPixel);
	if (NbPixel!=0)
	{
		e=e/NbPixel;
	}
	else
	{
		//Pas d'intersection: on met une valeur d'erreur moyenne
		e=0.05;
	}
	
	return e;
}

//
//Ajoute l'image passe en parametres  l'image compositee codee sur 16 bit
//
void AjouteImageCompositage(ImageRGBA & img)
{
	long x,y,x0,y0;
	
	for (y=0;y<ImageReference.hauteur;y++)
		for (x=0;x<ImageReference.largeur;x++)
		{
			x0=x+img.x0-ImageReference.x0;
			y0=y+img.y0-ImageReference.y0;
			if ((x0>=0) && (x0<img.largeur) && (y0>=0) && (y0<img.hauteur))
			{
				ImageCompositage[(x+y*ImageReference.largeur)*4]=ImageCompositage[(x+y*ImageReference.largeur)*4]+img.data[(x0+y0*img.largeur)*4]*img.data[(x0+y0*img.largeur)*4+3]/255;
				ImageCompositage[(x+y*ImageReference.largeur)*4+1]=ImageCompositage[(x+y*ImageReference.largeur)*4+1]+img.data[(x0+y0*img.largeur)*4+1]*img.data[(x0+y0*img.largeur)*4+3]/255;
				ImageCompositage[(x+y*ImageReference.largeur)*4+2]=ImageCompositage[(x+y*ImageReference.largeur)*4+2]+img.data[(x0+y0*img.largeur)*4+2]*img.data[(x0+y0*img.largeur)*4+3]/255;
				//sur le dernier plan on compte le nombre d'image qui ont t composites au pixel courant
				ImageCompositage[(x+y*ImageReference.largeur)*4+3]=ImageCompositage[(x+y*ImageReference.largeur)*4+3]+img.data[(x0+y0*img.largeur)*4+3]/255;
			}
		}
}

//
//Lecture des paramtres
//
void LectureScript(void)
{
	FILE *FichierParam;
	char Commande[255];	
	printf("Lecture des parametres\n");
	int i,iseq,CommandeValide;
	char *TitreCommande;
	char *p;
	char *NomOrigine,*NomDestination;
	long nb,DebutSequence,FinSequence;
	
	//Initialisation des paramtres par dfaut
	NomImageReference=NULL;
	NbrImagesTest=0;
	LargeurGrille=0;		//Permet de savoir s'il y a eu une commande de registration par blocs
	LargeurRecherche=0;
	DecalageMinX=0;
	DecalageMaxX=0;
	DecalageMinY=0;
	DecalageMaxY=0;
	DecalagePas=1;
	GlobalDecalageMinX=0;
	GlobalDecalageMaxX=0;
	GlobalDecalageMinY=0;
	GlobalDecalageMaxY=0;
	GlobalDecalagePas=1;
	ThetaMin=0;
	ThetaMax=0;
	ThetaPas=1;
	DimensionLibre=1;
	FractionPartieCentraleTranslation=6;
			
	if ((FichierParam=fopen(NomFichierScript,"r")) == NULL) 
	{
		printf("Impossible d'ouvrir le fichier %s\n",NomFichierScript);
		ErreurFatale();
	}
	else 
	{
		printf("\nFichier de script : %s\n\n\n",NomFichierScript);
	}
	
	i=0;
	while (feof(FichierParam)==0)
	{
		i++;
		CommandeValide=0;
		fgets(Commande,1000,FichierParam); 
		TitreCommande=strtok(Commande,": ");
		if (strcmp(TitreCommande,"\n")==0) CommandeValide=1;
		//printf("Ligne %i Commande %s\n",i,TitreCommande);
		//
		//Source:NomFichierSource,NomFichierResultat[,x0,y0]
		//
		if ((strcmp(TitreCommande,"Source")==0) || (strcmp(TitreCommande,"source")==0))
		{
			nb=NbrImagesTest+1;
			if (nb>NbrMaxImages)
			{
				printf("Erreur : trop d'images  traiter\n");
				ErreurFatale();
			}
			p=strtok(NULL,", \n");
			if (p!=NULL)
			{
				NomImageTest[nb]=new char[strlen(p)+1];
				strcpy(NomImageTest[nb],p);
				printf("Image source: %s ",NomImageTest[nb]);
			}
			else printf("Commande source sans nom de fichier source. Ligne %i\n",i);
			p=strtok(NULL,", \n");
			if (p!=NULL)
			{
				NomImageArrivee[nb]=new char[strlen(p)+1];
				strcpy(NomImageArrivee[nb],p);			
				printf("Fichier resultat: %s",NomImageArrivee[nb]);
			}
			else printf("Commande source sans nom de fichier resultat. Ligne %i\n",i);
			
			p=strtok(NULL,", \n");
			if (p!=NULL)
			{
				ImageTestX0[nb]=atol(p);
				printf (" Point x0=%i",ImageTestX0[nb]);
				TestX0Affecte[nb]=1;
			}
			else TestX0Affecte[nb]=0;
			p=strtok(NULL,", \n");
			if (p!=NULL)
			{
				ImageTestY0[nb]=atol(p);
				printf (" y0=%i",ImageTestY0[nb]);
				TestY0Affecte[nb]=1;
			}
			else TestY0Affecte[nb]=0;
			printf("\n");
			if ((NomImageTest[nb]!=NULL) && (NomImageArrivee[nb]!=NULL))
			{
				NbrImagesTest=nb;
			}
			CommandeValide=1;
		}
		//
		//sourcesbmp:nom generique des fichiers bmp,nom generique de sortie,debut de la sequence,fin de la sequence
		//
		if ((strcmp(TitreCommande,"Sourcesbmp")==0) || (strcmp(TitreCommande,"sourcesbmp")==0))
		{
			//Lecture des parametres de la ligne
			p=strtok(NULL,", \n");
			if (p!=NULL)
			{
				NomOrigine=new char[strlen(p)+1];
				strcpy(NomOrigine,p);
				printf("Nom generique de la sequence entree: %s   ",NomOrigine);
			}
			else printf("Commande source sans nom de fichier source. Ligne %i\n",i);
			p=strtok(NULL,", \n");
			if (p!=NULL)
			{
				NomDestination=new char[strlen(p)+1];
				strcpy(NomDestination,p);
				printf("sortie: %s   ",NomDestination);
			}
			else printf("Commande source sans nom de fichier de sortie. Ligne %i\n",i);
		
			p=strtok(NULL,", \n");
			if (p!=NULL)
			{
				DebutSequence=atol(p);
				printf ("Debut=%i  ",DebutSequence);
			}
			else printf("Pas de numero de debut");
			p=strtok(NULL,", \n");
			if (p!=NULL)
			{
				FinSequence=atol(p);
				printf ("Fin=%i\n",FinSequence);
			}
			else printf("Pas de numero de fin");
			
			//traduction
			if (FinSequence-DebutSequence+1+NbrImagesTest<NbrMaxImages)
			{
				for (iseq=DebutSequence;iseq<=FinSequence;iseq++)
				{
					nb=NbrImagesTest+1;
					NomImageTest[nb]=new char[strlen(NomOrigine)+1+10];
					sprintf(NomImageTest[nb],"%s%i%s",NomOrigine,iseq,".bmp");
					printf("source: %s -> ",NomImageTest[nb]);
					
					NomImageArrivee[nb]=new char[strlen(NomDestination)+1+10];
					sprintf(NomImageArrivee[nb],"%s%i%s",NomDestination,iseq,".bmp");
					printf("resultat: %s",NomImageArrivee[nb]);
					
					TestX0Affecte[nb]=0;
					TestY0Affecte[nb]=0;
					printf("\n");
					NbrImagesTest=nb;
				}
			}
			else
			{
				printf("Trop d'images  traiter\n");
				ErreurFatale();
			}
			CommandeValide=1;
		}
		//
		//Reference:nomfichierRef [,imgrefx0,imgrefy0]
		//
		if ((strcmp(TitreCommande,"Reference")==0) || (strcmp(TitreCommande,"reference")==0))
		{
			if (NomImageReference!=NULL) printf("Attention: plusieurs commande reference dans le mme fichier\n");
			p=strtok(NULL,", \n");
			if (p!=NULL)
			{
				NomImageReference=new char[strlen(p)+1];
				strcpy(NomImageReference,p);
				printf("Image de reference: %s",NomImageReference);
			}
			else printf("Commande reference sans nom de fichier. Ligne %i\n",i);
			
			p=strtok(NULL,", \n");
			if (p!=NULL)
			{
				ImageReferenceX0=atol(p);
				printf (" Point x0=%i",ImageReferenceX0);
				ReferenceX0Affecte=1;
			}
			else 
			{
				ImageReferenceX0=0;
				ReferenceX0Affecte=0;
			}
			p=strtok(NULL,", \n");
			if (p!=NULL)
			{
				ImageReferenceY0=atol(p);
				printf (" y0=%i",ImageReferenceY0);
				ReferenceY0Affecte=1;
			}
			else
			{
				ImageReferenceY0=0;
				ReferenceY0Affecte=0;
			}
			printf("\n");
			CommandeValide=1;
		}
		//
		//Compositage:nomfichierCompo
		//
		if ((strcmp(TitreCommande,"Compositage")==0) || (strcmp(TitreCommande,"compositage")==0))
		{
			p=strtok(NULL,", \n");
			if (p!=NULL)
			{
				NomImageCompositage=new char[strlen(p)+1];
				strcpy(NomImageCompositage,p);
				printf("Image compositee: %s",NomImageCompositage);
			}
			else printf("Commande compositage sans nom de fichier. Ligne %i\n",i);
			printf("\n");
			CommandeValide=1;
		}
		//
		//Dimension:libre ou fixe
		//
		if ((strcmp(TitreCommande,"Dimension")==0) || (strcmp(TitreCommande,"dimension")==0))
		{
			p=strtok(NULL,", \n");
			if (p!=NULL)
			{
				if ((strcmp(p,"Libre")==0) || (strcmp(p,"libre")==0))
				{
					printf("La taille de sortie des images peut tre differente de la taille d'origine");
					DimensionLibre=1;
				}
				if ((strcmp(p,"Libre")==0) || (strcmp(p,"libre")==0))
				{
					printf("La taille de sortie des images sera la meme que la taille d'origine");
					printf("\n ***** Pas encore implemente...");
					DimensionLibre=0;
				}
			}
			printf("\n");
			CommandeValide=1;
		}
		//
		//OptiTranslation:	FractionPartieCentraleTranslation
		//
		if ((strcmp(TitreCommande,"optitranslation")==0) || (strcmp(TitreCommande,"OptiTranslation")==0) || (strcmp(TitreCommande,"Optitranslation")==0) || (strcmp(TitreCommande,"optiTranslation")==0))
		{
			p=strtok(NULL,", \n");
			if (p!=NULL)
			{
				FractionPartieCentraleTranslation=atoi(p);
				if (FractionPartieCentraleTranslation<1)
				{
					FractionPartieCentraleTranslation=1;
				}
				printf ("Fraction de l'image prise en compte pour la recherche de translation=1/%i  ",FractionPartieCentraleTranslation);
			}
			printf("\n");
			CommandeValide=1;
		}

		//
		if ((strcmp(TitreCommande,"Dimension")==0) || (strcmp(TitreCommande,"dimension")==0))
		{
			p=strtok(NULL,", \n");
			if (p!=NULL)
			{
				if ((strcmp(p,"Libre")==0) || (strcmp(p,"libre")==0))
				{
					printf("La taille de sortie des images peut etre differente de la taille d'origine");
					DimensionLibre=1;
				}
				if ((strcmp(p,"Libre")==0) || (strcmp(p,"libre")==0))
				{
					printf("La taille de sortie des images sera la meme que la taille d'origine");
					printf("\n ***** Pas encore implment...");
					DimensionLibre=0;
				}
			}
			printf("\n");
			CommandeValide=1;
		}
		//
		//Angle:ThetaMin,ThetaMax,ThetaPas
		//
		if ((strcmp(TitreCommande,"Angle")==0) || (strcmp(TitreCommande,"angle")==0))
		{
			p=strtok(NULL,", \n");
			if (p!=NULL)
			{
				ThetaMin=atof(p);
				printf ("Angle min=%f  ",ThetaMin);
				ThetaMin=ThetaMin*3.141592654/180;
			}
			p=strtok(NULL,", \n");
			if (p!=NULL)
			{
				ThetaMax=atof(p);
				printf ("Angle max=%f  ",ThetaMax);
				ThetaMax=ThetaMax*3.141592654/180;
			}
			p=strtok(NULL,", \n");
			if (p!=NULL)
			{
				ThetaPas=atof(p);
				printf ("Resolution=%f  ",ThetaPas);
				ThetaPas=ThetaPas*3.141592654/180;
			}
			printf("\n");
			CommandeValide=1;
		}
		//
		//Translation:GlobalDecalageMax[,GlobalDecalagePas]
		//
		if ((strcmp(TitreCommande,"Translation")==0) || (strcmp(TitreCommande,"translation")==0))
		{
			p=strtok(NULL,", \n");
			if (p!=NULL)
			{
				GlobalDecalageMaxX=atoi(p);
				printf ("Decalage global maximum=%i  ",GlobalDecalageMaxX);
				GlobalDecalageMinX=-GlobalDecalageMaxX;
				GlobalDecalageMinY=-GlobalDecalageMaxX;
				GlobalDecalageMaxY=GlobalDecalageMaxX;			
			}
			p=strtok(NULL,", \n");
			if (p!=NULL)
			{
				GlobalDecalagePas=atoi(p);
				if (GlobalDecalagePas==0) GlobalDecalagePas=1; //Le pas de dcalage global doit tre entier
				printf (" pas=%i  ",GlobalDecalagePas);
			}
			printf("\n");
			CommandeValide=1;
		}
		//
		//Bloc:LargeurGrille,LargeurRecherche,DecalageMax [,DecalagePas=1]
		//
		if ((strcmp(TitreCommande,"Bloc")==0) || (strcmp(TitreCommande,"bloc")==0))
		{
			p=strtok(NULL,", \n");
			if (p!=NULL)
			{
				LargeurGrille=atoi(p);
				printf ("Largeur de grille=%i  ",LargeurGrille);
			}
			p=strtok(NULL,", \n");
			if (p!=NULL)
			{
				LargeurRecherche=atoi(p);
				printf ("Largeur de recherche=%i  ",LargeurRecherche);
			}
			p=strtok(NULL,", \n");
			if (p!=NULL)
			{
				DecalageMinX=-atoi(p);
				DecalageMaxX=-DecalageMinX;
				DecalageMinY=DecalageMinX;
				DecalageMaxY=-DecalageMinX;
				printf ("Decalage maxi de bloc=%i  ",DecalageMaxX);
			}
			p=strtok(NULL,", \n");
			if (p!=NULL)
			{
				DecalagePas=atof(p);
				printf ("Pas de decalage de bloc=%f  ",DecalagePas);
			}
			printf("\n");
			CommandeValide=1;
		}
		//
		//Commentaire
		//
		if ((strcmp(TitreCommande,"Rem")==0) || (strcmp(TitreCommande,"rem")==0))
		{
			CommandeValide=1;
		}
		//
		//Quitter
		//
		if ((strcmp(TitreCommande,"Quitter")==0) || (strcmp(TitreCommande,"quitter")==0))
		{
			OptionQuitter=1;
			CommandeValide=1;
		}
		//
		//Autre chose...
		//
		if (CommandeValide==0)
		{
			printf("*********** Commande inconnue ligne %i\n",i);
		}
		
	}

	fclose(FichierParam);

}


int ChargeBMP(ImageRGBA & img,const char * Nom)
{
	FILE *Fichier;
	long Taille;
        fpos_t post;
        fpos_t OffsetDonneesBrutest;
	long & pos=post.__pos;
	long & OffsetDonneesBrutes=OffsetDonneesBrutest.__pos;
	unsigned long l;
	unsigned short s;
	char a,b,c,d;
	char *Entete;
	long TailleEntete;
	int OctetsParPixel;
	
	long k;
	long IncrementLigne,TailleLigne;
	char *BufferLigne;
			
	printf("Chargement du fichier %s",Nom);
	if ((Fichier=fopen(Nom,"rb")) == NULL)
	{
		printf("Erreur : impossible d'ouvrir l'image %s\n",Nom);
		return 1;
	}
	else
	{
		//Lecture de l'entete
		if (Machine==0)
		{
			// *********************************************
			// Version PC
			// *********************************************
			//Taille de l'entete
			pos=10;
			fsetpos(Fichier,&post);
			Taille=4;
			fread(&OffsetDonneesBrutes,1,Taille,Fichier);
			//Mise en mmoire de l'entete pour la sauvegarde du rsultat
			Entete=new char[OffsetDonneesBrutes];
			if (Entete==NULL)
			{
				printf("Erreur : pas assez de memoire");
				ErreurFatale();
			}
			pos=0;
			fsetpos(Fichier,&post);
			Taille=OffsetDonneesBrutes;
			fread(Entete,1,Taille,Fichier);
			TailleEntete=OffsetDonneesBrutes;
			//Largeur et hauteur
			pos=18;
			fsetpos(Fichier,&post);
			Taille=4;
			fread(&l,1,Taille,Fichier);
			img.largeur=l;
			Taille=4;
			fread(&l,1,Taille,Fichier);
			img.hauteur=l;
			//Nombre de bits par pixel
			pos=28;
			fsetpos(Fichier,&post);
			Taille=2;
			fread(&s,1,Taille,Fichier);
			if (s==8) OctetsParPixel=1;
			if (s==24) OctetsParPixel=3;
			//On vrifie que l'image n'est pas compresse
			Taille=4;
			fread(&l,1,Taille,Fichier);
			if (l!=0)
			{
				printf("Fichier BMP compresse: chargement impossible\n");
				ErreurFatale();
			}
		}
		else
		{
			// *********************************************
			// Version Mac
			// *********************************************
			//Taille de l'entete
			pos=10;
			fsetpos(Fichier,&post);
			Taille=1;
			fread(&a,1,Taille,Fichier);
			Taille=1;
			fread(&b,1,Taille,Fichier);
			Taille=1;
			fread(&c,1,Taille,Fichier);
			Taille=1;
			fread(&d,1,Taille,Fichier);
			OffsetDonneesBrutes=d*256*256*256+c*256*256+b*256+a;
			//Mise en mmoire de l'entete pour la sauvegarde du rsultat
			Entete=new char[OffsetDonneesBrutes];
			if (Entete==NULL)
			{
				printf("Erreur : pas assez de memoire");
				ErreurFatale();
			}
			pos=0;
			fsetpos(Fichier,&post);
			Taille=OffsetDonneesBrutes;
			fread(Entete,1,Taille,Fichier);
			TailleEntete=OffsetDonneesBrutes;
			//Largeur et hauteur
			pos=18;
			fsetpos(Fichier,&post);
			Taille=1;
			fread(&a,1,Taille,Fichier);
			Taille=1;
			fread(&b,1,Taille,Fichier);
			Taille=1;
			fread(&c,1,Taille,Fichier);
			Taille=1;
			fread(&d,1,Taille,Fichier);
			img.largeur=d*256*256*256+c*256*256+b*256+a;
			Taille=1;
			fread(&a,1,Taille,Fichier);
			Taille=1;
			fread(&b,1,Taille,Fichier);
			Taille=1;
			fread(&c,1,Taille,Fichier);
			Taille=1;
			fread(&d,1,Taille,Fichier);
			img.hauteur=d*256*256*256+c*256*256+b*256+a;
			//Nombre de bits par pixel
			pos=28;
			fsetpos(Fichier,&post);
			Taille=1;
			fread(&a,1,Taille,Fichier);
			Taille=1;
			fread(&b,1,Taille,Fichier);
			s=b*256+a;
			if (s==8) OctetsParPixel=1;
			if (s==24) OctetsParPixel=3;
			//On vrifie que l'image n'est pas compresse
			Taille=1;
			fread(&a,1,Taille,Fichier);
			Taille=1;
			fread(&b,1,Taille,Fichier);
			Taille=1;
			fread(&c,1,Taille,Fichier);
			Taille=1;
			fread(&d,1,Taille,Fichier);
			l=d*256*256*256+c*256*256+b*256+a;
			if (l!=0)
			{
				printf("Fichier BMP compresse: chargement impossible\n");
				ErreurFatale();
			}
	
		}
	
		//printf("(e=%i  l=%i  h=%i  o=%i)",OffsetDonneesBrutes,img.largeur,img.hauteur,OctetsParPixel);
		
		//Accs aux donnes de l'image
		//Chargement ligne par ligne car les BMP ont toujours des lignes qui finissent sur un multiple de 32 bits
		TailleLigne=img.largeur*OctetsParPixel;
		IncrementLigne=(TailleLigne-1)/4;	//Taille d'une ligne dans le fichier=taille d'une ligne arrondie au multiple de 4 suprieur
		IncrementLigne=IncrementLigne*4+4;
		IncrementLigne=IncrementLigne-TailleLigne;	//Nombre d'octets  ajouter aprs la lecture d'une ligne
		BufferLigne=new char[TailleLigne+IncrementLigne];
		
		Taille=img.largeur*img.hauteur*4;
		img.data=new unsigned char[Taille];
		if (img.data==NULL)
		{
			printf("Erreur : pas assez de memoire \n");
			ErreurFatale();
		}
		
		fsetpos(Fichier,&OffsetDonneesBrutest);
		//Lecture des lignes une par une
		if (OctetsParPixel==1)
		{
			for (k=0;k<img.hauteur;k++)
			{
				for (l=0;l<img.largeur;l++) 
				{
				}
				fseek(Fichier,IncrementLigne,SEEK_CUR);
			}
			for (l=0;l<100;l++) printf("La sauvegarde en N&B n'est pas encore possible\n");
		}
		else if (OctetsParPixel==3)
		{
			for (k=0;k<img.hauteur;k++)
			{
				Taille=TailleLigne+IncrementLigne;
				fread(BufferLigne,1,Taille,Fichier);
				for (l=0;l<img.largeur;l++) 
				{
					img.data[( (img.hauteur-k-1) *img.largeur+l)*4]=BufferLigne[l*OctetsParPixel];
					img.data[( (img.hauteur-k-1) *img.largeur+l)*4+1]=BufferLigne[l*OctetsParPixel+1];
					img.data[( (img.hauteur-k-1) *img.largeur+l)*4+2]=BufferLigne[l*OctetsParPixel+2];
					img.data[( (img.hauteur-k-1) *img.largeur+l)*4+3]=255;	//Opaque
				}
			}		
		}
		else
		{
			printf("Erreur fatale: mode d'image non supporte\n");
			ErreurFatale();
		}
		fclose(Fichier);
		delete [] BufferLigne;
		return 0;
	}	
	printf(" FIN\n");
	//Initialisation des champs de l'image
	img.x0=0;
	img.y0=0;
	delete [] Entete;	
}


void EcritCommandeFits(FILE *f,const char *com,int n)
{
	char v[80];
	int i;
	
	fprintf(f,com);
	for (i=0;i<8-strlen(com);i++) fprintf(f," ");
	fprintf(f,"=");	
	sprintf(v,"%i",n);
	for (i=0;i<30-strlen(v)-9;i++) fprintf(f," ");
	fprintf(f,v);
	for (i=0;i<80-30;i++) fprintf(f," ");
}
void EcritCommandeFits(FILE *f,const char *com,char c)
{
	char v[80];
	int i;
	
	fprintf(f,com);
	for (i=0;i<8-strlen(com);i++) fprintf(f," ");
	fprintf(f,"=");	
	sprintf(v,"%c",c);
	for (i=0;i<30-strlen(v)-9;i++) fprintf(f," ");
	fprintf(f,v);
	for (i=0;i<80-30;i++) fprintf(f," ");
}
void EcritCommandeFitsEND(FILE *f)
{
	fpos_t pos;
	
	fprintf(f,"END");
	fgetpos(f,&pos);
	while(pos.__pos<2880)
	{
		fprintf(f," ");
		fgetpos(f,&pos);
	}
}

void SauveImageCompositage(const char * Nom,int canal)
{
	FILE *Fichier;
	int a,x,y;
	float nbdiv,nbmul;
	unsigned char octet1,octet2;

	if (NbrImagesTest<127) nbmul=NbrImagesTest;
	else nbmul=127;
	
	Fichier=fopen(Nom,"wb");
	EcritCommandeFits(Fichier,"SIMPLE",'T');
	EcritCommandeFits(Fichier,"BITPIX",16);
	EcritCommandeFits(Fichier,"NAXIS",2);
	EcritCommandeFits(Fichier,"NAXIS1",(int) ImageReference.largeur);
	EcritCommandeFits(Fichier,"NAXIS2",(int) ImageReference.hauteur);
	EcritCommandeFits(Fichier,"MIPS-HI",(int) (256*nbmul));
	EcritCommandeFits(Fichier,"MIPS-LO",0);
	EcritCommandeFitsEND(Fichier);
	
	for (y=ImageReference.hauteur-1;y>=0;y--)
	{
		for (x=0;x<ImageReference.largeur;x++)
		{
			nbdiv=NbrImagesTest+0*ImageCompositage[(x+y*ImageReference.largeur)*4+3];
			a=ImageCompositage[(x+y*ImageReference.largeur)*4+canal]*nbmul/nbdiv;
			octet1=a/256;
			octet2=a-octet1*256;
			fwrite(&octet1,1,1,Fichier);		
			fwrite(&octet2,1,1,Fichier);		
		}
	}
	
	fclose(Fichier);
}

//Inversion des octets de poids fort et faible pour criture de fichers BMP sur Mac
void InverseOctets4(unsigned long &l)
{
	char a,b,c,d;

	char *tab=(char *) &l;
	a=tab[0];
	b=tab[1];
	c=tab[2];
	d=tab[3];	
	l=d*256*256*256+c*256*256+b*256+a;
}

void InverseOctets2(unsigned int &i)
{
	unsigned char a,b;
	
	a=i/256;
	b=i%256;
	
	i=b*256+a;
}

void SauveBMP(const ImageRGBA &img,const char * Nom)
{
	FILE *Fichier;
	long Taille;
	long k,i,l;
	long TailleLigne,IncrementLigne;
	char *BufferLigne;
	int OctetsParPixel=3;	//c'est une image RGB qui sera sauvegarde.
	
	unsigned long lb;
	unsigned int wb;
	unsigned char cb;
	
	printf("Sauvegarde d'une image %s",Nom);
	Fichier=fopen(Nom,"wb");

	//Ecriture de l'entete
	if (Machine==0)
	{
		// *********************************************
		// Version PC
		// *********************************************
		
		//header partie 1
		cb='B';
		fwrite(&cb,1,1,Fichier);
		cb='M';
		fwrite(&cb,1,1,Fichier);
		l=img.largeur/4;
		l=l*4+4;
		lb=54+l*img.hauteur*3;		//Taille totale du fichier
		fwrite(&lb,1,4,Fichier);
		wb=0;						//2 mots  0
		fwrite(&wb,1,2,Fichier);
		fwrite(&wb,1,2,Fichier);
		lb=54;						//Offset pour atteindre les donnes
		fwrite(&lb,1,4,Fichier);
		//header partie 2(BitmapInfoHeader)
		lb=40;						//longueur du BitmapInfoHeader
		fwrite(&lb,1,4,Fichier);
		lb=img.largeur;
		fwrite(&lb,1,4,Fichier);
		lb=img.hauteur;
		fwrite(&lb,1,4,Fichier);
		wb=1;						//1 plan
		fwrite(&wb,1,2,Fichier);
		wb=24;						//bits par pixel
		fwrite(&wb,1,2,Fichier);
		lb=0;						//Non compress
		fwrite(&lb,1,4,Fichier);
		lb=0;
		fwrite(&lb,1,4,Fichier);
		lb=2840;					//Nombre de pixels par metre (horizontal puis vertical)
		fwrite(&lb,1,4,Fichier);
		fwrite(&lb,1,4,Fichier);
		lb=0;
		fwrite(&lb,1,4,Fichier);
		fwrite(&lb,1,4,Fichier);
	}
	else
	{
		// *********************************************
		// Version Mac
		// *********************************************
		
		//header partie 1
		cb='B';
		fwrite(&cb,1,1,Fichier);
		cb='M';
		fwrite(&cb,1,1,Fichier);
		l=img.largeur/4;
		l=l*4+4;
		lb=54+l*img.hauteur*3;		//Taille totale du fichier
		InverseOctets4(lb);
		fwrite(&lb,1,4,Fichier);
		wb=0;
		InverseOctets2(wb);						//2 mots  0
		fwrite(&wb,1,2,Fichier);
		fwrite(&wb,1,2,Fichier);
		lb=54;						//Offset pour atteindre les donnes
		InverseOctets4(lb);
		fwrite(&lb,1,4,Fichier);
		//header partie 2(BitmapInfoHeader)
		lb=40;						//longueur du BitmapInfoHeader
		InverseOctets4(lb);
		fwrite(&lb,1,4,Fichier);
		lb=img.largeur;
		InverseOctets4(lb);
		fwrite(&lb,1,4,Fichier);
		lb=img.hauteur;
		InverseOctets4(lb);
		fwrite(&lb,1,4,Fichier);
		wb=1;						//1 plan
		InverseOctets2(wb);
		fwrite(&wb,1,2,Fichier);
		wb=24;						//bits par pixel
		InverseOctets2(wb);
		fwrite(&wb,1,2,Fichier);
		lb=0;						//Non compress
		InverseOctets4(lb);
		fwrite(&lb,1,4,Fichier);
		lb=0;
		InverseOctets4(lb);		
		fwrite(&lb,1,4,Fichier);
		lb=2840;					//Nombre de pixels par metre (horizontal puis vertical)
		InverseOctets4(lb);
		fwrite(&lb,1,4,Fichier);
		fwrite(&lb,1,4,Fichier);
		lb=0;
		InverseOctets4(lb);
		fwrite(&lb,1,4,Fichier);
		fwrite(&lb,1,4,Fichier);
	}
	
	//Ecriture de l'image ligne par ligne
	TailleLigne=img.largeur*OctetsParPixel;
	IncrementLigne=(TailleLigne-1)/4;	//Taille d'une ligne dans le fichier=taille d'une ligne arrondie au multiple de 4 suprieur
	IncrementLigne=IncrementLigne*4+4;
	IncrementLigne=IncrementLigne-TailleLigne;	//Nombre d'octets  ajouter aprs la lecture d'une ligne

	BufferLigne=new char[TailleLigne+IncrementLigne];
	for (k=0;k<img.hauteur;k++)
	{
		//Copie des donnes de la ligne avec supression de l'octet de transparence
		for (l=0;l<img.largeur;l++)
		{
			BufferLigne[l*3]=img.data[( (img.hauteur-k-1) *img.largeur+l)*4];
			BufferLigne[l*3+1]=img.data[( (img.hauteur-k-1) *img.largeur+l)*4+1];
			BufferLigne[l*3+2]=img.data[( (img.hauteur-k-1) *img.largeur+l)*4+2];
		}
		for (i=TailleLigne;i<(TailleLigne+IncrementLigne);i++)	//on complte par des zros
		{
			BufferLigne[i]=0;
		}
		Taille=TailleLigne+IncrementLigne;
		fwrite(BufferLigne,1,Taille,Fichier);
	}
	
	delete [] BufferLigne;
	fclose(Fichier);
	printf(" FIN\n");
}

//Arret du programme pour cause d'erreur
void ErreurFatale(void)
{
	char c;
	
	//Une erreur a provoqu l'arrt du programme
	printf("Erreur fatale, appuyez sur entree.\n");
	printf("Fatal error. Press return.\n");
	c = getchar();
	abort();
}

//Fonctions utiles
template <class T>
T max(const T & a,const T & b)
{
	if (a<=b) return b;
	else return a;
}

template <class T>
T min(const T & a,const T & b)
{
	if (a>=b) return b;
	else return a;
}
